#!/usr/bin/env python3

"""Domain.services.infrastructure_integration_services
Where: Domain service module providing reusable business logic.
What: Houses service routines and helpers implemented in this module.
Why: Makes this service behaviour accessible to application workflows.
"""

from __future__ import annotations

"""インフラ統合関連ドメインサービス集

InfrastructureIntegrationUseCaseから分離された専門ロジックを格納
"""


from dataclasses import dataclass
from typing import Any

from noveler.domain.entities.infrastructure_integration_aggregate import (
    BatchExecutionResult,
    InfrastructureIntegrationAggregate,
    ServiceExecutionResult,
    ServiceHealthStatus,
)
from noveler.domain.services.infrastructure_coordination_service import InfrastructureCoordinationService
from noveler.domain.value_objects.infrastructure_configuration import (
    CacheConfiguration,
    InfrastructureConfiguration,
    PerformanceConfiguration,
    ServiceConfiguration,
)


@dataclass(frozen=True)
class RegisterServiceRequest:
    """サービス登録要求"""

    project_id: str
    service_config: ServiceConfiguration
    override_existing: bool = False


@dataclass
class RegisterServiceResponse:
    """サービス登録応答"""

    success: bool
    service_id: str | None = None
    service_name: str | None = None
    validation_errors: list[str] = None
    error_message: str | None = None


@dataclass(frozen=True)
class ExecuteServiceRequest:
    """サービス実行要求"""

    project_id: str
    service_name: str
    execution_context: dict[str, Any]
    use_fallback: bool = True
    cache_enabled: bool = True


@dataclass
class ExecuteServiceResponse:
    """サービス実行応答"""

    success: bool
    execution_result: ServiceExecutionResult | None = None
    error_message: str | None = None


@dataclass(frozen=True)
class HealthCheckRequest:
    """健康状態チェック要求"""

    project_id: str
    service_names: list[str] | None = None  # None の場合は全サービス


@dataclass
class HealthCheckResponse:
    """健康状態チェック応答"""

    success: bool
    health_statuses: list[ServiceHealthStatus] = None
    overall_health: str = "unknown"  # "healthy", "degraded", "critical"
    error_message: str | None = None


@dataclass(frozen=True)
class BatchExecutionRequest:
    """バッチ実行要求"""

    project_id: str
    batch_jobs: list[dict[str, Any]]
    parallel_execution: bool = True
    max_concurrency: int = 5


@dataclass
class BatchExecutionResponse:
    """バッチ実行応答"""

    success: bool
    batch_result: BatchExecutionResult | None = None
    error_message: str | None = None


class InfrastructureServiceRegistrationService:
    """インフラサービス登録専用サービス"""

    def __init__(self, coordination_service: InfrastructureCoordinationService) -> None:
        self.coordination_service = coordination_service

    def register_service(
        self,
        aggregate: InfrastructureIntegrationAggregate,
        request: RegisterServiceRequest,
    ) -> dict[str, Any]:
        """サービスを登録"""
        try:
            # サービス設定の検証
            existing_configs = [service.configuration for service in aggregate.service_registry.get_all_services()]
            validation_errors = self.coordination_service.validate_service_configuration(
                request.service_config,
                [ServiceConfiguration(**config) for config in existing_configs],
            )

            if validation_errors:
                return {"success": False, "validation_errors": validation_errors}

            # 既存サービスとの重複チェック
            if not request.override_existing:
                if aggregate.is_service_registered(request.service_config.name):
                    return {
                        "success": False,
                        "error": f"サービス '{request.service_config.name}' は既に登録されています",
                    }

            # サービスの登録
            service_config_dict = {
                "service_type": request.service_config.service_type.value,
                "name": request.service_config.name,
                "adapter_class": request.service_config.adapter_class,
                "enabled": request.service_config.enabled,
                "priority": request.service_config.priority,
                "timeout": request.service_config.timeout,
                "retry_count": request.service_config.retry_count,
                "fallback_service": request.service_config.fallback_service,
                "dependencies": request.service_config.dependencies,
                "cache_enabled": request.service_config.cache_enabled,
                "cache_key_template": request.service_config.cache_key_template,
                "batch_size": request.service_config.batch_size,
                "health_check_interval": request.service_config.health_check_interval,
            }

            service = aggregate.register_service(service_config_dict)

            return {
                "success": True,
                "service": service,
                "service_id": service.service_id,
                "service_name": service.name,
            }

        except Exception as e:
            return {"success": False, "error": f"サービス登録エラー: {e!s}"}


class InfrastructureServiceExecutionService:
    """インフラサービス実行専用サービス"""

    def execute_service(
        self,
        aggregate: InfrastructureIntegrationAggregate,
        request: ExecuteServiceRequest,
    ) -> dict[str, Any]:
        """サービスを実行"""
        try:
            # サービスの存在確認
            if not aggregate.is_service_registered(request.service_name):
                return {
                    "success": False,
                    "error": f"サービス '{request.service_name}' が見つかりません",
                }

            # 実行コンテキストの準備
            execution_context = request.execution_context.copy()
            execution_context["project_id"] = request.project_id
            execution_context["cache_enabled"] = request.cache_enabled

            # サービス実行(フォールバック考慮)
            if request.use_fallback:
                execution_result = aggregate.execute_service_with_fallback(
                    request.service_name,
                    execution_context,
                )

            else:
                execution_result = aggregate.execute_service(
                    request.service_name,
                    execution_context,
                )

            return {"success": True, "execution_result": execution_result}

        except Exception as e:
            return {"success": False, "error": f"サービス実行エラー: {e!s}"}


class InfrastructureHealthCheckService:
    """インフラヘルスチェック専用サービス"""

    def check_health(
        self,
        aggregate: InfrastructureIntegrationAggregate,
        request: HealthCheckRequest,
    ) -> dict[str, Any]:
        """サービスの健康状態をチェック"""
        try:
            # 対象サービスの特定
            if request.service_names:
                target_services = request.service_names
            else:
                # 全サービスを対象
                target_services = [service.name for service in aggregate.service_registry.get_all_services()]

            # 各サービスの健康状態チェック
            health_statuses = []
            for service_name in target_services:
                if aggregate.is_service_registered(service_name):
                    status = aggregate.check_service_health(service_name)
                    health_statuses.append(status)

            # 全体的な健康状態の評価
            overall_health = self._evaluate_overall_health(health_statuses)

            return {
                "success": True,
                "health_statuses": health_statuses,
                "overall_health": overall_health,
            }

        except Exception as e:
            return {"success": False, "error": f"ヘルスチェックエラー: {e!s}"}

    def _evaluate_overall_health(self, health_statuses: list[ServiceHealthStatus]) -> str:
        """全体的な健康状態を評価"""
        if not health_statuses:
            return "unknown"

        critical_count = sum(1 for status in health_statuses if not status.is_healthy)
        degraded_count = sum(1 for status in health_statuses if status.is_healthy and status.response_time > 5000)

        total_services = len(health_statuses)
        critical_ratio = critical_count / total_services
        degraded_ratio = degraded_count / total_services

        if critical_ratio > 0.5:
            return "critical"
        if critical_ratio > 0.1 or degraded_ratio > 0.3:
            return "degraded"
        return "healthy"


class InfrastructureBatchExecutionService:
    """インフラバッチ実行専用サービス"""

    def execute_batch(
        self,
        aggregate: InfrastructureIntegrationAggregate,
        request: BatchExecutionRequest,
    ) -> dict[str, Any]:
        """バッチ実行"""
        try:
            # バッチジョブの検証
            validation_errors = self._validate_batch_jobs(request.batch_jobs)
            if validation_errors:
                return {"success": False, "validation_errors": validation_errors}

            # バッチ実行
            batch_result = aggregate.execute_batch(
                request.batch_jobs,
                parallel=request.parallel_execution,
                max_concurrency=request.max_concurrency,
            )

            return {"success": True, "batch_result": batch_result}

        except Exception as e:
            return {"success": False, "error": f"バッチ実行エラー: {e!s}"}

    def _validate_batch_jobs(self, batch_jobs: list[dict[str, Any]]) -> list[str]:
        """バッチジョブを検証"""
        errors: list[Any] = []
        for i, job in enumerate(batch_jobs):
            if "service_name" not in job:
                errors.append(f"ジョブ {i}: service_name が必要です")
            if "context" not in job:
                errors.append(f"ジョブ {i}: context が必要です")
            if not isinstance(job.get("context"), dict):
                errors.append(f"ジョブ {i}: context は辞書である必要があります")
        return errors


class InfrastructureStatisticsService:
    """インフラ統計情報専用サービス"""

    def get_service_statistics(self, aggregate: InfrastructureIntegrationAggregate) -> dict[str, Any]:
        """サービス統計情報を取得"""
        try:
            statistics = aggregate.get_service_statistics()
            return {"success": True, "statistics": statistics}
        except Exception as e:
            return {"success": False, "error": f"統計情報取得エラー: {e!s}"}


class InfrastructureConfigurationService:
    """インフラ設定管理専用サービス"""

    def __init__(self, default_config: InfrastructureConfiguration | None = None) -> None:
        self.default_config = default_config or self._create_default_config()

    def get_configuration(self, aggregate: InfrastructureIntegrationAggregate) -> dict[str, Any]:
        """設定情報を取得"""
        try:
            config = aggregate.get_configuration()
            return {"success": True, "configuration": config}
        except Exception as e:
            return {"success": False, "error": f"設定取得エラー: {e!s}"}

    def _create_default_config(self) -> InfrastructureConfiguration:
        """デフォルト設定を作成"""
        cache_config: dict[str, Any] = CacheConfiguration(
            enabled=True,
            max_size=1000,
            ttl_seconds=3600,
            key_prefix="infra_",
        )

        performance_config: dict[str, Any] = PerformanceConfiguration(
            max_concurrent_executions=10,
            default_timeout=30,
            health_check_interval=60,
            retry_backoff_factor=2.0,
        )

        return InfrastructureConfiguration(
            cache_configuration=cache_config,
            performance_configuration=performance_config,
            enable_metrics=True,
            enable_logging=True,
        )


class InfrastructureAggregateManagementService:
    """インフラアグリゲート管理専用サービス"""

    def __init__(self, default_config: InfrastructureConfiguration) -> None:
        self.default_config = default_config
        self._aggregates: dict[str, InfrastructureIntegrationAggregate] = {}

    def get_or_create_aggregate(self, project_id: str) -> InfrastructureIntegrationAggregate:
        """アグリゲートを取得または作成"""
        if project_id not in self._aggregates:
            self._aggregates[project_id] = InfrastructureIntegrationAggregate(
                project_id=project_id,
                configuration=self.default_config,
            )

        return self._aggregates[project_id]

    def get_aggregate(self, project_id: str) -> InfrastructureIntegrationAggregate | None:
        """アグリゲートを取得"""
        return self._aggregates.get(project_id)



#!/usr/bin/env python3
"""テストレポート生成ツール

TDD+DDD準拠のテスト結果を分析し、詳細なレポートを生成する
"""

import json
import subprocess
import sys
from pathlib import Path

from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


class TestReportGenerator:
    """テストレポート生成器"""

    def __init__(self, project_root: Path | str) -> None:
        self.project_root = project_root
        self.report_dir = project_root / "logs" / "reports"
        self.report_dir.mkdir(parents=True, exist_ok=True)

    def generate_comprehensive_report(self) -> dict[str, object]:
        """包括的なテストレポートを生成"""

        report = {
            "timestamp": project_now().datetime.isoformat(),
            "project_root": str(self.project_root),
            "test_results": {},
            "coverage_report": {},
            "quality_metrics": {},
            "performance_metrics": {},
            "compliance_check": {},
            "summary": {},
        }

        # テスト結果の収集
        report["test_results"] = self._collect_test_results()

        # カバレッジレポートの生成
        report["coverage_report"] = self._generate_coverage_report()

        # 品質メトリクスの収集
        report["quality_metrics"] = self._collect_quality_metrics()

        # パフォーマンスメトリクスの収集
        report["performance_metrics"] = self._collect_performance_metrics()

        # TDD+DDD準拠チェック
        report["compliance_check"] = self._check_tdd_ddd_compliance()

        # サマリーの生成
        report["summary"] = self._generate_summary(report)

        # レポートファイルの保存
        self._save_report(report)

        return report

    def _collect_test_results(self) -> dict[str, object]:
        """テスト結果の収集"""
        test_results = {
            "unit_tests": {},
            "integration_tests": {},
            "e2e_tests": {},
            "total_tests": 0,
            "passed_tests": 0,
            "failed_tests": 0,
            "skipped_tests": 0,
            "errors": [],
        }

        try:
            # ユニットテストの実行
            unit_result = self._run_tests("tests/unit", "unit")
            test_results["unit_tests"] = unit_result

            # 統合テストの実行
            integration_result = self._run_tests("tests/integration", "integration")
            test_results["integration_tests"] = integration_result

            # E2Eテストの実行
            e2e_result = self._run_tests("tests/e2e", "e2e")
            test_results["e2e_tests"] = e2e_result

            # 統計の計算
            for test_type in ["unit_tests", "integration_tests", "e2e_tests"]:
                result = test_results[test_type]
                if result:
                    test_results["total_tests"] += result.get("total", 0)
                    test_results["passed_tests"] += result.get("passed", 0)
                    test_results["failed_tests"] += result.get("failed", 0)
                    test_results["skipped_tests"] += result.get("skipped", 0)

        except Exception as e:
            test_results["errors"].append(f"テスト結果収集エラー: {e}")

        return test_results

    def _run_tests(self, test_path: str, test_type: str) -> dict[str, object]:
        """指定されたテストを実行"""
        result = {
            "test_type": test_type,
            "total": 0,
            "passed": 0,
            "failed": 0,
            "skipped": 0,
            "execution_time": 0,
            "details": [],
        }

        try:
            # pytest実行
            cmd = [
                "python",
                "-m",
                "pytest",
                test_path,
                "-v",
                "--tb=short",
                "--json-report",
                f"--json-report-file=logs/{test_type}_report.json",
            ]

            start_time = project_now().datetime
            process = subprocess.run(
                cmd,
                cwd=self.project_root,
                capture_output=True,
                text=True,
                timeout=300,
                check=False,
            )

            end_time = project_now().datetime

            result["execution_time"] = (end_time - start_time).total_seconds()
            result["return_code"] = process.returncode
            result["stdout"] = process.stdout
            result["stderr"] = process.stderr

            # JSONレポートの解析
            json_report_path = self.project_root / "logs" / f"{test_type}_report.json"
            if json_report_path.exists():
                with Path(json_report_path).open(encoding="utf-8") as f:
                    json_report = json.load(f)
                    summary = json_report.get("summary", {})
                    result["total"] = summary.get("total", 0)
                    result["passed"] = summary.get("passed", 0)
                    result["failed"] = summary.get("failed", 0)
                    result["skipped"] = summary.get("skipped", 0)
                    result["details"] = json_report.get("tests", [])

        except subprocess.TimeoutExpired:
            result["error"] = "テスト実行がタイムアウトしました"
        except Exception as e:
            result["error"] = f"テスト実行エラー: {e!s}"

        return result

    def _generate_coverage_report(self) -> dict[str, object]:
        """カバレッジレポートの生成"""
        coverage_report = {
            "overall_coverage": 0,
            "domain_coverage": 0,
            "application_coverage": 0,
            "infrastructure_coverage": 0,
            "quality_coverage": 0,
            "files": [],
            "missing_lines": {},
            "branch_coverage": 0,
        }

        try:
            # カバレッジ測定の実行
            cmd = [
                "python",
                "-m",
                "pytest",
                "tests/unit",
                "--cov=domain",
                "--cov=application",
                "--cov=infrastructure",
                "--cov=quality",
                "--cov=utils",
                "--cov-report=json:logs/coverage.json",
                "--cov-report=xml:logs/coverage.xml",
                "--cov-report=html:logs/htmlcov",
                "--cov-report=term-missing",
                "--tb=no",
                "-q",
            ]

            subprocess.run(
                cmd,
                cwd=self.project_root,
                capture_output=True,
                text=True,
                timeout=300,
                check=False,
            )

            # JSONカバレッジレポートの解析
            coverage_json_path = self.project_root / "logs" / "coverage.json"
            if coverage_json_path.exists():
                with Path(coverage_json_path).open(encoding="utf-8") as f:
                    coverage_data = json.load(f)

                    # 全体カバレッジ
                    totals = coverage_data.get("totals", {})
                    coverage_report["overall_coverage"] = totals.get("percent_covered", 0)

                    # ファイル別カバレッジ
                    files = coverage_data.get("files", {})
                    for file_path, file_data in files.items():
                        file_info = {
                            "file": file_path,
                            "coverage": file_data.get("summary", {}).get("percent_covered", 0),
                            "lines_covered": file_data.get("summary", {}).get("covered_lines", 0),
                            "lines_total": file_data.get("summary", {}).get("num_statements", 0),
                            "missing_lines": file_data.get("missing_lines", []),
                        }
                        coverage_report["files"].append(file_info)

                        # 層別カバレッジの計算
                        if "domain/" in file_path:
                            coverage_report["domain_coverage"] = max(
                                coverage_report["domain_coverage"],
                                file_info["coverage"],
                            )

                        elif "application/" in file_path:
                            coverage_report["application_coverage"] = max(
                                coverage_report["application_coverage"],
                                file_info["coverage"],
                            )

                        elif "infrastructure/" in file_path:
                            coverage_report["infrastructure_coverage"] = max(
                                coverage_report["infrastructure_coverage"],
                                file_info["coverage"],
                            )

                        elif "quality/" in file_path:
                            coverage_report["quality_coverage"] = max(
                                coverage_report["quality_coverage"],
                                file_info["coverage"],
                            )

        except Exception as e:
            coverage_report["error"] = f"カバレッジレポート生成エラー: {e!s}"

        return coverage_report

    def _collect_quality_metrics(self) -> dict[str, object]:
        """品質メトリクスの収集"""
        quality_metrics = {
            "code_quality": {},
            "type_checking": {},
            "security": {},
            "documentation": {},
            "complexity": {},
        }

        try:
            # flake8による品質チェック
            quality_metrics["code_quality"] = self._run_flake8()

            # mypyによる型チェック
            quality_metrics["type_checking"] = self._run_mypy()

            # banditによるセキュリティチェック
            quality_metrics["security"] = self._run_bandit()

            # pydocstyleによるドキュメンテーションチェック
            quality_metrics["documentation"] = self._run_pydocstyle()

        except Exception as e:
            quality_metrics["error"] = f"品質メトリクス収集エラー: {e!s}"

        return quality_metrics

    def _run_flake8(self) -> dict[str, object]:
        """flake8による品質チェック"""
        try:
            cmd = ["python", "-m", "flake8", "domain", "application", "infrastructure", "quality", "--format=json"]
            result = subprocess.run(cmd, cwd=self.project_root, capture_output=True, text=True, check=False)

            violations = []
            if result.stdout:
                try:
                    violations = json.loads(result.stdout)
                except json.JSONDecodeError:
                    violations = []

            return {
                "violations": violations,
                "total_violations": len(violations),
                "return_code": result.returncode,
            }
        except Exception as e:
            return {"error": str(e)}

    def _run_mypy(self) -> dict[str, object]:
        """mypyによる型チェック"""
        try:
            cmd = ["python", "-m", "mypy", "domain", "application", "--ignore-missing-imports"]
            result = subprocess.run(cmd, cwd=self.project_root, capture_output=True, text=True, check=False)

            errors = []
            if result.stdout:
                lines = result.stdout.strip().split("\n")
                errors.extend(line.strip() for line in lines if "error:" in line)

            return {
                "errors": errors,
                "total_errors": len(errors),
                "return_code": result.returncode,
            }
        except Exception as e:
            return {"error": str(e)}

    def _run_bandit(self) -> dict[str, object]:
        """banditによるセキュリティチェック"""
        try:
            cmd = ["python", "-m", "bandit", "-r", "domain", "application", "infrastructure", "quality", "-f", "json"]
            result = subprocess.run(cmd, cwd=self.project_root, capture_output=True, text=True, check=False)

            security_issues = []
            if result.stdout:
                try:
                    bandit_report = json.loads(result.stdout)
                    security_issues = bandit_report.get("results", [])
                except json.JSONDecodeError:
                    security_issues = []

            return {
                "security_issues": security_issues,
                "total_issues": len(security_issues),
                "return_code": result.returncode,
            }
        except Exception as e:
            return {"error": str(e)}

    def _run_pydocstyle(self) -> dict[str, object]:
        """pydocstyleによるドキュメンテーションチェック"""
        try:
            cmd = ["python", "-m", "pydocstyle", "domain", "application", "--convention=google"]
            result = subprocess.run(cmd, cwd=self.project_root, capture_output=True, text=True, check=False)

            doc_issues = []
            if result.stdout:
                lines = result.stdout.strip().split("\n")
                doc_issues.extend(line.strip() for line in lines if line.strip())

            return {
                "documentation_issues": doc_issues,
                "total_issues": len(doc_issues),
                "return_code": result.returncode,
            }
        except Exception as e:
            return {"error": str(e)}

    def _collect_performance_metrics(self) -> dict[str, object]:
        """パフォーマンスメトリクスの収集"""
        performance_metrics = {
            "test_execution_time": {},
            "memory_usage": {},
            "benchmark_results": {},
        }

        try:
            # ベンチマークテストの実行
            cmd = [
                "python",
                "-m",
                "pytest",
                "tests/unit",
                "-k",
                "performance",
                "--benchmark-only",
                "--benchmark-json=logs/benchmark.json",
            ]

            subprocess.run(
                cmd,
                cwd=self.project_root,
                capture_output=True,
                text=True,
                timeout=300,
                check=False,
            )

            # ベンチマーク結果の解析
            benchmark_path = self.project_root / "logs" / "benchmark.json"
            if benchmark_path.exists():
                with Path(benchmark_path).open(encoding="utf-8") as f:
                    benchmark_data = json.load(f)
                    performance_metrics["benchmark_results"] = benchmark_data

        except Exception as e:
            performance_metrics["error"] = f"パフォーマンスメトリクス収集エラー: {e!s}"

        return performance_metrics

    def _check_tdd_ddd_compliance(self) -> dict[str, object]:
        """TDD+DDD準拠チェック"""
        compliance_check = {
            "tdd_compliance": {},
            "ddd_compliance": {},
            "overall_compliance": False,
        }

        try:
            # TDD+DDD準拠チェックツールの実行
            cmd = ["python", "tools/check_tdd_ddd_compliance.py"]
            result = subprocess.run(
                cmd,
                cwd=self.project_root,
                capture_output=True,
                text=True,
                check=False,
            )

            compliance_check["return_code"] = result.returncode
            compliance_check["overall_compliance"] = result.returncode == 0
            compliance_check["output"] = result.stdout
            compliance_check["errors"] = result.stderr

        except Exception as e:
            compliance_check["error"] = f"準拠チェックエラー: {e!s}"

        return compliance_check

    def _generate_summary(self, report: dict[str, object]) -> dict[str, object]:
        """サマリーの生成"""
        summary = {
            "overall_status": "PASS",
            "test_success_rate": 0,
            "coverage_percentage": 0,
            "quality_score": 0,
            "compliance_status": "FAIL",
            "recommendations": [],
        }

        try:
            # テスト成功率の計算
            test_results = report.get("test_results", {})
            total_tests = test_results.get("total_tests", 0)
            passed_tests = test_results.get("passed_tests", 0)

            if total_tests > 0:
                summary["test_success_rate"] = (passed_tests / total_tests) * 100

            # カバレッジ率の取得
            coverage_report = report.get("coverage_report", {})
            summary["coverage_percentage"] = coverage_report.get("overall_coverage", 0)

            # 品質スコアの計算
            quality_metrics = report.get("quality_metrics", {})
            quality_score = 100

            # flake8エラーによる減点
            code_quality = quality_metrics.get("code_quality", {})
            violations = code_quality.get("total_violations", 0)
            quality_score -= min(violations * 2, 30)

            # mypyエラーによる減点
            type_checking = quality_metrics.get("type_checking", {})
            type_errors = type_checking.get("total_errors", 0)
            quality_score -= min(type_errors * 3, 30)

            # セキュリティ問題による減点
            security = quality_metrics.get("security", {})
            security_issues = security.get("total_issues", 0)
            quality_score -= min(security_issues * 5, 20)

            summary["quality_score"] = max(quality_score, 0)

            # 準拠ステータスの取得
            compliance_check = report.get("compliance_check", {})
            summary["compliance_status"] = "PASS" if compliance_check.get("overall_compliance", False) else "FAIL"

            # 総合ステータスの判定
            if (
                summary["test_success_rate"] >= 90
                and summary["coverage_percentage"] >= 80
                and summary["quality_score"] >= 80
                and summary["compliance_status"] == "PASS"
            ):
                summary["overall_status"] = "PASS"
            else:
                summary["overall_status"] = "FAIL"

            # 推奨事項の生成
            recommendations = []

            if summary["test_success_rate"] < 90:
                recommendations.append("テスト成功率を90%以上に改善してください")

            if summary["coverage_percentage"] < 80:
                recommendations.append("テストカバレッジを80%以上に改善してください")

            if summary["quality_score"] < 80:
                recommendations.append("コード品質を改善してください(flake8、mypy、banditの警告を解消)")

            if summary["compliance_status"] == "FAIL":
                recommendations.append("TDD+DDD準拠チェックをパスしてください")

            summary["recommendations"] = recommendations

        except Exception as e:
            summary["error"] = f"サマリー生成エラー: {e!s}"

        return summary

    def _save_report(self, report: dict[str, object]) -> None:
        """レポートファイルの保存"""
        try:
            # JSONレポートの保存
            json_path = self.report_dir / "test_report.json"
            with Path(json_path).open("w", encoding="utf-8") as f:
                json.dump(report, f, indent=2, ensure_ascii=False)

            # HTMLレポートの生成
            html_path = self.report_dir / "test_report.html"
            self._generate_html_report(report, html_path)

            # Markdownレポートの生成
            md_path = self.report_dir / "test_report.md"
            self._generate_markdown_report(report, md_path)


        except Exception:
            pass

    def _generate_html_report(self, report: dict[str, object], output_path: Path | str) -> None:
        """HTMLレポートの生成"""
        summary = report.get("summary", {})

        html_content = f"""
        <!DOCTYPE html>
        <html>
        <head>
            <title>TDD+DDD テストレポート</title>
            <meta charset="UTF-8">
            <style>
                body {{ font-family: Arial, sans-serif; margin: 20px; }}
                .header {{ background-color: #f0f0f0; padding: 20px; border-radius: 5px; }}
                .section {{ margin: 20px 0; padding: 15px; border: 1px solid #ddd; border-radius: 5px; }}
                .pass {{ color: green; }}
                .fail {{ color: red; }}
                .warning {{ color: orange; }}
                table {{ border-collapse: collapse; width: 100%; }}
                th, td {{ border: 1px solid #ddd; padding: 8px; text-align: left; }}
                th {{ background-color: #f2f2f2; }}
            </style>
        </head>
        <body>
            <div class="header">
                <h1>TDD+DDD テストレポート</h1>
                <p>生成日時: {report.get("timestamp", "Unknown")}</p>
                <p>総合ステータス: <span class="{"pass" if summary.get("overall_status") == "PASS" else "fail"}">{summary.get("overall_status", "Unknown")}</span></p>
            </div>

            <div class="section">
                <h2>サマリー</h2>
                <table>
                    <tr><th>項目</th><th>値</th></tr>
                    <tr><td>テスト成功率</td><td>{summary.get("test_success_rate", 0):.1f}%</td></tr>
                    <tr><td>カバレッジ率</td><td>{summary.get("coverage_percentage", 0):.1f}%</td></tr>
                    <tr><td>品質スコア</td><td>{summary.get("quality_score", 0):.1f}</td></tr>
                    <tr><td>TDD+DDD準拠</td><td>{summary.get("compliance_status", "Unknown")}</td></tr>
                </table>
            </div>

            <div class="section">
                <h2>テスト結果</h2>
                <p>実装中...</p>
            </div>

            <div class="section">
                <h2>推奨事項</h2>
                <ul>
        """

        for recommendation in summary.get("recommendations", []):
            html_content += f"<li>{recommendation}</li>"

        html_content += """
                </ul>
            </div>
        </body>
        </html>
        """

        with Path(output_path).open("w", encoding="utf-8") as f:
            f.write(html_content)

    def _generate_markdown_report(self, report: dict[str, object], output_path: Path | str) -> None:
        """Markdownレポートの生成"""
        summary = report.get("summary", {})

        md_content = f"""# TDD+DDD テストレポート

**生成日時**: {report.get("timestamp", "Unknown")}
**総合ステータス**: {summary.get("overall_status", "Unknown")}

## サマリー

| 項目 | 値 |
|------|-----|
| テスト成功率 | {summary.get("test_success_rate", 0):.1f}% |
| カバレッジ率 | {summary.get("coverage_percentage", 0):.1f}% |
| 品質スコア | {summary.get("quality_score", 0):.1f} |
| TDD+DDD準拠 | {summary.get("compliance_status", "Unknown")} |

## テスト結果

### ユニットテスト
- 実装中...

### 統合テスト
- 実装中...

### E2Eテスト
- 実装中...

## 推奨事項

"""

        for recommendation in summary.get("recommendations", []):
            md_content += f"- {recommendation}\n"

        md_content += """
## 詳細情報

完全な詳細情報は `test_report.json` を参照してください。
"""

        with Path(output_path).open("w", encoding="utf-8") as f:
            f.write(md_content)


def main() -> None:
    """メイン実行関数"""
    project_root = Path(__file__).parent.parent

    generator = TestReportGenerator(project_root)
    report = generator.generate_comprehensive_report()

    # サマリーの表示
    summary = report.get("summary", {})

    if summary.get("recommendations"):
        for _rec in summary["recommendations"]:
            pass

    if summary.get("overall_status") == "PASS":
        sys.exit(0)
    else:
        sys.exit(1)


if __name__ == "__main__":
    main()



# File: src/noveler/domain/services/ml_quality/ml_quality_optimizer.py
# Purpose: ML-based quality optimization orchestrator service
# Context: Integrates corpus learning, dynamic thresholds, weight optimization, and severity estimation

"""
ML Quality Optimizer Service.

This service orchestrates ML-enhanced quality evaluation by coordinating:
- Corpus-based baseline metrics
- Dynamic threshold adjustment
- Weight optimization
- Context-aware severity estimation

Architecture:
- Domain Service (pure business logic, no infrastructure dependencies)
- Depends on other domain services via dependency injection
- Returns domain entities and value objects only

Contract: SPEC-QUALITY-140 §2.2.1
"""

from dataclasses import dataclass
from enum import Enum
from pathlib import Path

from noveler.domain.interfaces.logger_interface import ILogger, NullLogger


class LearningMode(Enum):
    """Learning mode for ML optimization."""
    ONLINE = "online"      # Incremental learning
    BATCH = "batch"        # Full retraining
    DISABLED = "disabled"  # No learning


@dataclass(frozen=True)
class ProjectContext:
    """Project metadata for ML optimization."""
    project_root: Path
    genre: str
    target_audience: str
    episode_count: int
    avg_quality_score: float | None = None


@dataclass(frozen=True)
class OptimizedQualityResult:
    """
    Result of ML-enhanced quality evaluation.

    Fields:
        base_scores: Standard quality scores (rhythm, readability, grammar, style)
        optimized_scores: ML-adjusted scores
        dynamic_thresholds: Context-aware thresholds learned from corpus
        severity_adjustments: Position/frequency-based severity modifications
        improvement_priorities: Ranked improvement suggestions
        ml_metadata: Debugging and explainability metadata
    """
    base_scores: dict[str, float]
    optimized_scores: dict[str, float]
    dynamic_thresholds: dict[str, dict]
    severity_adjustments: list[dict]
    improvement_priorities: list[dict]
    ml_metadata: dict


class MLQualityOptimizer:
    """
    ML-based quality optimization orchestrator.

    Responsibilities:
    - Integrate corpus learning, dynamic thresholds, and feedback loops
    - Coordinate weight optimization and severity estimation
    - Generate adaptive quality recommendations

    Dependencies (injected):
    - corpus_analyzer: CorpusAnalyzer for baseline metrics
    - threshold_adjuster: DynamicThresholdAdjuster for learned thresholds
    - weight_optimizer: WeightOptimizer for aspect weights
    - severity_estimator: SeverityEstimator for context-aware penalties
    - logger: ILogger for diagnostics

    Contract: SPEC-QUALITY-140 §2.2.1
    """

    def __init__(
        self,
        corpus_analyzer: "CorpusAnalyzer",
        threshold_adjuster: "DynamicThresholdAdjuster",
        weight_optimizer: "WeightOptimizer",
        severity_estimator: "SeverityEstimator",
        logger: ILogger | None = None
    ):
        """
        Initialize ML quality optimizer.

        Args:
            corpus_analyzer: Service for corpus-based baseline metrics
            threshold_adjuster: Service for dynamic threshold adjustment
            weight_optimizer: Service for weight optimization
            severity_estimator: Service for severity estimation
            logger: Optional logger (defaults to NullLogger)
        """
        self._corpus_analyzer = corpus_analyzer
        self._threshold_adjuster = threshold_adjuster
        self._weight_optimizer = weight_optimizer
        self._severity_estimator = severity_estimator
        self._logger = logger or NullLogger()

    def optimize_quality_evaluation(
        self,
        manuscript: str,
        project_context: ProjectContext,
        base_quality_scores: dict[str, float],
        base_issues: list[dict],
        learning_mode: LearningMode = LearningMode.ONLINE
    ) -> OptimizedQualityResult:
        """
        Execute ML-enhanced quality evaluation.

        Args:
            manuscript: Target manuscript text
            project_context: Project metadata (genre, target_audience, etc.)
            base_quality_scores: Standard quality scores from existing checks
            base_issues: List of quality issues from existing checks
            learning_mode: ONLINE (incremental), BATCH (full retrain), or DISABLED

        Returns:
            OptimizedQualityResult with:
            - base_scores: Original quality scores
            - optimized_scores: ML-adjusted scores
            - dynamic_thresholds: Context-aware thresholds
            - severity_adjustments: Position/frequency-based adjustments
            - improvement_priorities: Ranked improvement suggestions

        Raises:
            MLOptimizationError: If optimization fails critically

        Algorithm:
            1. Extract corpus baseline metrics (genre-specific)
            2. Adjust thresholds based on corpus + feedback history
            3. Optimize aspect weights using historical performance
            4. Re-score issues with context-aware severity
            5. Generate prioritized improvement suggestions

        Performance:
            - Target: ≤10 seconds total (5s ML overhead + 5s standard checks)
            - Corpus metrics: ≤3s (cached: ≤100ms)
            - Weight optimization: ≤2s
            - Severity estimation: ≤100ms per issue

        Contract: SPEC-QUALITY-140 §4, §11
        """
        self._logger.info(
            "Starting ML-enhanced quality evaluation",
            extra={
                "genre": project_context.genre,
                "learning_mode": learning_mode.value
            }
        )

        if learning_mode == LearningMode.DISABLED:
            self._logger.info("ML optimization disabled, returning base scores")
            return OptimizedQualityResult(
                base_scores=base_quality_scores,
                optimized_scores=base_quality_scores,
                dynamic_thresholds={},
                severity_adjustments=[],
                improvement_priorities=[],
                ml_metadata={"mode": "disabled"}
            )

        # Step 1: Get corpus-based baseline metrics
        self._logger.debug("Fetching corpus baseline metrics")
        corpus_metrics = self._corpus_analyzer.build_baseline_metrics(
            genre=project_context.genre,
            target_audience=project_context.target_audience,
            project_root=project_context.project_root
        )

        # Step 2: Adjust thresholds dynamically
        self._logger.debug("Adjusting thresholds based on feedback history")
        dynamic_thresholds = self._threshold_adjuster.adjust_thresholds(
            current_thresholds=self._extract_current_thresholds(base_quality_scores),
            corpus_metrics=corpus_metrics,
            project_root=project_context.project_root
        )

        # Step 3: Optimize weights
        self._logger.debug("Optimizing aspect weights")
        optimized_weights = self._weight_optimizer.optimize_weights(
            project_root=project_context.project_root,
            base_scores=base_quality_scores
        )

        # Step 4: Re-estimate severity with context
        self._logger.debug("Re-estimating issue severity with context")
        severity_adjustments = []
        for issue in base_issues:
            adjusted = self._severity_estimator.estimate_severity(
                issue=issue,
                manuscript=manuscript,
                total_lines=manuscript.count("\n") + 1
            )
            severity_adjustments.append(adjusted)

        # Step 5: Recalculate optimized scores
        optimized_scores = self._recalculate_scores(
            base_scores=base_quality_scores,
            weights=optimized_weights,
            severity_adjustments=severity_adjustments
        )

        # Step 6: Generate improvement priorities
        improvement_priorities = self._generate_priorities(
            severity_adjustments=severity_adjustments,
            optimized_scores=optimized_scores,
            dynamic_thresholds=dynamic_thresholds
        )

        self._logger.info(
            "ML optimization complete",
            extra={
                "base_overall": base_quality_scores.get("overall", 0),
                "optimized_overall": optimized_scores.get("overall", 0),
                "priority_count": len(improvement_priorities)
            }
        )

        return OptimizedQualityResult(
            base_scores=base_quality_scores,
            optimized_scores=optimized_scores,
            dynamic_thresholds=dynamic_thresholds,
            severity_adjustments=severity_adjustments,
            improvement_priorities=improvement_priorities,
            ml_metadata={
                "mode": learning_mode.value,
                "corpus_sample_count": corpus_metrics.get("sample_count", 0),
                "weights_optimized": True,
                "thresholds_adjusted": True
            }
        )

    def _extract_current_thresholds(
        self,
        base_scores: dict[str, float]
    ) -> dict[str, float]:
        """
        Extract current thresholds from base scores.

        This is a placeholder implementation. In production, thresholds
        should be read from .novelerrc.yaml or project configuration.
        """
        return {
            "overall": 80.0,
            "rhythm": 75.0,
            "readability": 70.0,
            "grammar": 85.0,
            "style": 75.0
        }

    def _recalculate_scores(
        self,
        base_scores: dict[str, float],
        weights: dict[str, float],
        severity_adjustments: list[dict]
    ) -> dict[str, float]:
        """
        Recalculate quality scores using optimized weights and adjusted severities.

        Algorithm:
            1. Apply optimized weights to each aspect score
            2. Adjust overall score based on re-estimated severity
            3. Ensure scores remain in [0, 100] range
        """
        # Placeholder: Simple weighted average
        # TODO: Integrate severity adjustments into score calculation
        weighted_sum = sum(
            base_scores.get(aspect, 0) * weights.get(aspect, 0)
            for aspect in weights
        )
        weight_total = sum(weights.values())

        optimized_overall = weighted_sum / weight_total if weight_total > 0 else 0

        optimized_scores = base_scores.copy()
        optimized_scores["overall"] = max(0.0, min(100.0, optimized_overall))

        return optimized_scores

    def _generate_priorities(
        self,
        severity_adjustments: list[dict],
        optimized_scores: dict[str, float],
        dynamic_thresholds: dict[str, dict]
    ) -> list[dict]:
        """
        Generate prioritized improvement suggestions.

        Algorithm:
            1. Identify aspects below dynamic thresholds
            2. Sort by (severity × impact) descending
            3. Return top N suggestions
        """
        priorities = []

        for aspect, score in optimized_scores.items():
            if aspect == "overall":
                continue

            threshold_data = dynamic_thresholds.get(aspect, {})
            threshold = threshold_data.get("threshold", 80.0)

            if score < threshold:
                gap = threshold - score
                priorities.append({
                    "aspect": aspect,
                    "current_score": score,
                    "target_threshold": threshold,
                    "gap": gap,
                    "priority": gap * 1.0  # Simple priority: larger gap = higher priority
                })

        # Sort by priority descending
        priorities.sort(key=lambda x: x["priority"], reverse=True)

        # Return top 5
        return priorities[:5]


# Type hints for forward references
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from noveler.domain.services.ml_quality.corpus_analyzer import CorpusAnalyzer
    from noveler.domain.services.ml_quality.dynamic_threshold_adjuster import DynamicThresholdAdjuster
    from noveler.domain.services.ml_quality.severity_estimator import SeverityEstimator
    from noveler.domain.services.ml_quality.weight_optimizer import WeightOptimizer



"""
エラーロガー実装（Infrastructure層 - Imperative Shell）
SPEC-ERR-001: 統一エラーハンドリングシステム

実際のログ出力を行う（副作用あり）
"""

import json
from pathlib import Path

from noveler.domain.services.error_classifier import ErrorLevel
from noveler.domain.value_objects.project_time import project_now
from noveler.infrastructure.logging.unified_logger import get_logger


class ErrorLogger:
    """
    エラーログ出力実装

    Imperative Shell:
    - ファイルへの書き込み（副作用）
    - 外部システムへの通知（副作用）
    """

    def __init__(self, log_file: Path | None = None, format: str = "text") -> None:
        """
        Args:
            log_file: ログファイルパス（オプション）
            format: ログフォーマット ("text" or "json")
        """
        self.log_file = log_file
        self.format = format
        self._setup_logger()

    def _setup_logger(self) -> None:
        """ロガーセットアップ（副作用）"""
        try:
            self.logger = get_logger("ErrorLogger")
        except Exception:

            class _Stub:
                def __init__(self):
                    import sys as _sys
                    self._stderr = _sys.stderr
                def log(self, *_args, **_kwargs):
                    pass
                def error(self, msg, *a, **k):
                    try:
                        self._stderr.write(str(msg)+"\n")
                    except Exception:
                        pass
                def warning(self, msg, *a, **k):
                    try:
                        self._stderr.write(str(msg)+"\n")
                    except Exception:
                        pass
                def info(self, *a, **k):
                    pass
                def debug(self, *a, **k):
                    pass
                def setLevel(self, *_):
                    pass
                def addHandler(self, *_):
                    pass
            self.logger = _Stub()

        if self.log_file:
            # 出力先ディレクトリが存在しない場合は作成
            try:
                self.log_file.parent.mkdir(parents=True, exist_ok=True)
            except Exception:
                pass

    def log(self, error: Exception, level: ErrorLevel, context: dict | None = None):
        """
        エラーをログに記録（副作用）

        Args:
            error: エラーオブジェクト
            level: エラーレベル
            context: 追加コンテキスト
        """
        if self.format == "json":
            log_entry = self.format_log_entry(error, level, context)
            self._write_json_log(log_entry)
        else:
            log_message = self._format_text_message(error, level, context)
            self._write_text_log(log_message, level)

    def format_log_entry(self, error: Exception, level: ErrorLevel,
                         context: dict | None = None) -> str:
        """
        構造化ログエントリーを生成

        Args:
            error: エラーオブジェクト
            level: エラーレベル
            context: 追加コンテキスト

        Returns:
            str: JSON形式のログエントリー
        """
        entry = {
            "timestamp": project_now().datetime.isoformat(),
            "level": level.value.upper(),
            "error_type": type(error).__name__,
            "message": str(error),
            "context": context or {}
        }

        return json.dumps(entry, ensure_ascii=False, default=str)

    def _format_text_message(self, error: Exception, level: ErrorLevel,
                            context: dict | None = None) -> str:
        """テキスト形式のメッセージ生成"""
        message = f"[{level.value.upper()}] {type(error).__name__}: {error}"

        if context:
            message += f" | Context: {context}"

        return message

    def _write_json_log(self, log_entry: str) -> None:
        """JSONログ書き込み（副作用）"""
        if self.log_file:
            with self.log_file.open("a", encoding="utf-8") as f:
                f.write(log_entry + "\n")

    def _write_text_log(self, message: str, level: ErrorLevel) -> None:
        """テキストログ書き込み（副作用）"""
        if self.log_file:
            try:
                with self.log_file.open("a", encoding="utf-8") as f:
                    f.write(f"{message}\n")
                return
            except Exception:
                pass
        # ロガーへ委譲（レベル名で分岐）
        method = {
            ErrorLevel.CRITICAL: "critical",
            ErrorLevel.ERROR: "error",
            ErrorLevel.WARNING: "warning",
            ErrorLevel.INFO: "info",
        }.get(level, "error")
        try:
            getattr(self.logger, method)(message)
        except Exception:
            try:
                # 最低限stderrへ
                import sys as _sys
                _sys.stderr.write(str(message)+"\n")
            except Exception:
                pass



"""エピソードとプロジェクトリポジトリインターフェーステスト"""

from abc import ABC
from datetime import datetime, timedelta
from unittest.mock import Mock

import pytest

from noveler.domain.entities.episode import Episode
from noveler.domain.repositories.episode_repository import (
    AdvancedEpisodeRepository,
    EpisodeQuery,
    EpisodeRepository,
)
from noveler.domain.repositories.project_repository import ProjectRepository
from noveler.domain.value_objects.project_time import project_now

pytestmark = pytest.mark.plot_episode


class TestEpisodeRepository:
    """エピソードリポジトリインターフェースのテスト"""
    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_interface_definition(self) -> None:
        """インターフェースが正しく定義されていることを確認"""
        assert issubclass(EpisodeRepository, ABC)

        # 全メソッドの存在確認
        methods = [
            "save",
            "find_by_id",
            "find_by_project_and_number",
            "find_all_by_project",
            "find_by_status",
            "find_by_date_range",
            "delete",
            "get_next_episode_number",
            "get_episode_count",
            "get_total_word_count",
            "find_by_tags",
            "find_by_quality_score_range",
            "find_ready_for_publication",
            "get_statistics",
            "bulk_update_status",
            "backup_episode",
            "restore_episode",
        ]
        for method in methods:
            assert hasattr(EpisodeRepository, method)

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_abstract_methods(self) -> None:
        """抽象メソッドの定義を確認"""
        with pytest.raises(TypeError, match=".*"):
            EpisodeRepository()

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_method_signatures(self) -> None:
        """メソッドシグネチャの確認"""
        # from noveler.domain.entities.episode import Episode  # Moved to top-level

        # モックエピソードの作成
        mock_episode = Mock(spec=Episode)
        mock_episode.id = "episode001"
        mock_episode.episode_number = 1
        mock_episode.status = "draft"
        mock_episode.word_count = 3000
        mock_episode.quality_score = 85.0
        mock_episode.tags = ["アクション", "バトル"]

        # モック実装を作成
        class MockEpisodeRepo(EpisodeRepository):
            def __init__(self) -> None:
                self.episodes = {}

            def save(self, episode: Episode, _project_id: str) -> None:
                key = f"{_project_id}:{episode.id}"
                self.episodes[key] = episode

            def find_by_id(self, _episode_id: str, _project_id: str) -> Episode | None:
                key = f"{_project_id}:{_episode_id}"
                return self.episodes.get(key)

            def find_by_project_and_number(self, _project_id: str, episode_number: int) -> Episode | None:
                for key, ep in self.episodes.items():
                    if key.startswith(f"{_project_id}:") and ep.episode_number == episode_number:
                        return ep
                return None

            def find_all_by_project(self, project_id: str) -> list[Episode]:
                return [ep for key, ep in self.episodes.items() if key.startswith(f"{project_id}:")]

            def find_by_status(self, project_id: str, status: str) -> list[Episode]:
                return [
                    ep for key, ep in self.episodes.items() if key.startswith(f"{project_id}:") and ep.status == status
                ]

            def find_by_date_range(self, _project_id: str, _start_date: datetime, _end_date: datetime) -> list[Episode]:
                # 実装例
                return []

            def delete(self, _episode_id: str, _project_id: str) -> bool:
                key = f"{_project_id}:{_episode_id}"
                if key in self.episodes:
                    del self.episodes[key]
                    return True
                return False

            def get_next_episode_number(self, project_id: str) -> int:
                project_episodes = self.find_all_by_project(project_id)
                if not project_episodes:
                    return 1
                return max(ep.episode_number for ep in project_episodes) + 1

            def get_episode_count(self, project_id: str) -> int:
                return len(self.find_all_by_project(project_id))

            def get_total_word_count(self, project_id: str) -> int:
                return sum(ep.word_count for ep in self.find_all_by_project(project_id))

            def find_by_tags(self, project_id: str, tags: list[str]) -> list[Episode]:
                result = []
                for key, ep in self.episodes.items():
                    if key.startswith(f"{project_id}:") and any(tag in ep.tags for tag in tags):
                        result.append(ep)
                return result

            def find_by_quality_score_range(self, project_id: str, min_score: float, max_score: float) -> list[Episode]:
                return [
                    ep
                    for key, ep in self.episodes.items()
                    if key.startswith(f"{project_id}:") and min_score <= ep.quality_score <= max_score
                ]

            def find_ready_for_publication(self, project_id: str) -> list[Episode]:
                return [
                    ep
                    for key, ep in self.episodes.items()
                    if key.startswith(f"{project_id}:") and ep.status == "ready_for_publication"
                ]

            def get_statistics(self, project_id: str) -> dict[str, object]:
                episodes = self.find_all_by_project(project_id)
                return {
                    "total_episodes": len(episodes),
                    "total_word_count": sum(ep.word_count for ep in episodes),
                    "average_quality_score": sum(ep.quality_score for ep in episodes) / len(episodes)
                    if episodes
                    else 0,
                }

            def bulk_update_status(self, project_id: str, episode_ids: list[str], new_status: str) -> int:
                count = 0
                for episode_id in episode_ids:
                    key = f"{project_id}:{episode_id}"
                    if key in self.episodes:
                        self.episodes[key].status = new_status
                        count += 1
                return count

            def backup_episode(self, _episode_id: str, _project_id: str) -> bool:
                return True  # 実装例

            def restore_episode(self, _episode_id: str, _project_id: str, _backup_version: str) -> bool:
                return True  # 実装例

        repo = MockEpisodeRepo()

        # 各メソッドのテスト
        repo.save(mock_episode, "test_project")
        assert repo.find_by_id("episode001", "test_project") == mock_episode
        assert repo.find_by_id("not_exists", "test_project") is None

        found = repo.find_by_project_and_number("test_project", 1)
        assert found == mock_episode

        all_episodes = repo.find_all_by_project("test_project")
        assert len(all_episodes) == 1

        assert repo.get_next_episode_number("test_project") == 2
        assert repo.get_episode_count("test_project") == 1
        assert repo.get_total_word_count("test_project") == 3000

        by_tags = repo.find_by_tags("test_project", ["アクション"])
        assert len(by_tags) == 1

        by_score = repo.find_by_quality_score_range("test_project", 80.0, 90.0)
        assert len(by_score) == 1

        stats = repo.get_statistics("test_project")
        assert stats["total_episodes"] == 1
        assert stats["average_quality_score"] == 85.0

        assert repo.delete("episode001", "test_project") is True
        assert repo.find_by_id("episode001", "test_project") is None


class TestEpisodeQuery:
    """エピソード検索クエリのテスト"""

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_query_builder(self) -> None:
        """クエリビルダーの動作確認"""
        query = EpisodeQuery()

        # メソッドチェーン
        query = (
            query.with_project("test_project")
            .with_episode_numbers([1, 2, 3])
            .with_statuses(["draft", "published"])
            .with_tags(["バトル", "ファンタジー"])
            .with_word_count_range(1000, 5000)
            .with_quality_score_range(70.0, 100.0)
            .order_by_field("created_at", desc=True)
            .with_pagination(limit=10, offset=20)
        )

        assert query.project_id == "test_project"
        assert query.episode_numbers == [1, 2, 3]
        assert query.statuses == ["draft", "published"]
        assert query.tags == ["バトル", "ファンタジー"]
        assert query.min_word_count == 1000
        assert query.max_word_count == 5000
        assert query.min_quality_score == 70.0
        assert query.max_quality_score == 100.0
        assert query.order_by == "created_at"
        assert query.order_desc is True
        assert query.limit == 10
        assert query.offset == 20

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_date_range_query(self) -> None:
        """日付範囲クエリのテスト"""
        query = EpisodeQuery()
        now = project_now().datetime
        yesterday = now - timedelta(days=1)
        tomorrow = now + timedelta(days=1)

        query = query.with_created_date_range(yesterday, tomorrow)
        assert query.created_after == yesterday
        assert query.created_before == tomorrow

        query = query.with_updated_date_range(yesterday, tomorrow)
        assert query.updated_after == yesterday
        assert query.updated_before == tomorrow


class TestAdvancedEpisodeRepository:
    """高度なエピソードリポジトリインターフェースのテスト"""

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_interface_definition(self) -> None:
        """インターフェースが正しく定義されていることを確認"""
        assert issubclass(AdvancedEpisodeRepository, EpisodeRepository)
        assert hasattr(AdvancedEpisodeRepository, "find_by_query")
        assert hasattr(AdvancedEpisodeRepository, "count_by_query")

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_inheritance(self) -> None:
        """継承関係の確認"""
        # AdvancedEpisodeRepositoryはEpisodeRepositoryのすべてのメソッドを持つ
        base_methods = [attr for attr in dir(EpisodeRepository) if not attr.startswith("_")]
        for method in base_methods:
            assert hasattr(AdvancedEpisodeRepository, method)


class TestProjectRepository:
    """プロジェクトリポジトリインターフェースのテスト"""

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_interface_definition(self) -> None:
        """インターフェースが正しく定義されていることを確認"""
        assert issubclass(ProjectRepository, ABC)

        # 全メソッドの存在確認
        methods = [
            "exists",
            "create",
            "get_project_info",
            "update_project_info",
            "delete",
            "get_all_projects",
            "get_project_settings",
            "update_project_settings",
            "get_project_metadata",
            "set_project_metadata",
            "archive_project",
            "restore_project",
            "get_project_statistics",
            "backup_project",
            "get_project_directory",
            "validate_project_structure",
            "initialize_project_structure",
            "get_project_root",
        ]
        for method in methods:
            assert hasattr(ProjectRepository, method)

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_abstract_methods(self) -> None:
        """抽象メソッドの定義を確認"""
        with pytest.raises(TypeError, match=".*"):
            ProjectRepository()

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_method_signatures(self) -> None:
        """メソッドシグネチャの確認"""

        # モック実装を作成
        class MockProjectRepo(ProjectRepository):
            def __init__(self) -> None:
                self.projects = {}
                self.settings = {}
                self.metadata = {}

            def exists(self, project_id: str) -> bool:
                return project_id in self.projects

            def create(self, project_id: str, project_data: dict[str, object]) -> bool:
                if project_id not in self.projects:
                    self.projects[project_id] = project_data
                    return True
                return False

            def get_project_info(self, project_id: str) -> dict[str, object] | None:
                return self.projects.get(project_id)

            def update_project_info(self, project_id: str, project_data: dict[str, object]) -> bool:
                if project_id in self.projects:
                    self.projects[project_id].update(project_data)
                    return True
                return False

            def delete(self, project_id: str) -> bool:
                if project_id in self.projects:
                    del self.projects[project_id]
                    return True
                return False

            def get_all_projects(self) -> list[dict[str, object]]:
                return list(self.projects.values())

            def get_project_settings(self, project_id: str) -> dict[str, object] | None:
                return self.settings.get(project_id)

            def update_project_settings(self, project_id: str, settings: dict[str, object]) -> bool:
                self.settings[project_id] = settings
                return True

            def get_project_metadata(self, project_id: str) -> dict[str, object] | None:
                return self.metadata.get(project_id)

            def set_project_metadata(self, project_id: str, key: str, value: object) -> bool:
                if project_id not in self.metadata:
                    self.metadata[project_id] = {}
                self.metadata[project_id][key] = value
                return True

            def archive_project(self, project_id: str) -> bool:
                if project_id in self.projects:
                    self.projects[project_id]["archived"] = True
                    return True
                return False

            def restore_project(self, project_id: str) -> bool:
                if project_id in self.projects:
                    self.projects[project_id]["archived"] = False
                    return True
                return False

            def get_project_statistics(self, project_id: str) -> dict[str, object] | None:
                if project_id in self.projects:
                    return {"episode_count": 0, "word_count": 0, "quality_score": 0.0}
                return None

            def backup_project(self, project_id: str) -> bool:
                return project_id in self.projects

            def get_project_directory(self, project_id: str) -> str | None:
                if project_id in self.projects:
                    return f"/projects/{project_id}"
                return None

            def validate_project_structure(self, project_id: str) -> dict[str, object]:
                return {"valid": project_id in self.projects, "errors": [], "warnings": []}

            def initialize_project_structure(self, project_id: str) -> bool:
                return project_id in self.projects

            def get_project_root(self, project_id: str) -> str:
                return f"/root/{project_id}"

        repo = MockProjectRepo()

        # 各メソッドのテスト
        assert repo.exists("test_project") is False

        project_data = {"name": "テストプロジェクト", "author": "作者名"}
        assert repo.create("test_project", project_data) is True
        assert repo.exists("test_project") is True

        info = repo.get_project_info("test_project")
        assert info is not None
        assert info["name"] == "テストプロジェクト"

        assert repo.update_project_info("test_project", {"genre": "ファンタジー"}) is True
        info = repo.get_project_info("test_project")
        assert "genre" in info

        all_projects = repo.get_all_projects()
        assert len(all_projects) == 1

        assert repo.update_project_settings("test_project", {"auto_save": True}) is True
        settings = repo.get_project_settings("test_project")
        assert settings is not None
        assert settings["auto_save"] is True

        assert repo.set_project_metadata("test_project", "version", "1.0.0") is True
        metadata = repo.get_project_metadata("test_project")
        assert metadata is not None
        assert metadata["version"] == "1.0.0"

        assert repo.archive_project("test_project") is True
        assert repo.restore_project("test_project") is True

        stats = repo.get_project_statistics("test_project")
        assert stats is not None
        assert "episode_count" in stats

        assert repo.backup_project("test_project") is True

        directory = repo.get_project_directory("test_project")
        assert directory == "/projects/test_project"

        validation = repo.validate_project_structure("test_project")
        assert validation["valid"] is True

        assert repo.initialize_project_structure("test_project") is True

        root = repo.get_project_root("test_project")
        assert root == "/root/test_project"

        assert repo.delete("test_project") is True
        assert repo.exists("test_project") is False


class TestRepositoryIntegration:
    """リポジトリインターフェースの統合テスト"""

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_repositories_follow_ddd_principles(self) -> None:
        """リポジトリがDDD原則に従っていることを確認"""
        repositories = [
            EpisodeRepository,
            AdvancedEpisodeRepository,
            ProjectRepository,
        ]

        for repo in repositories:
            assert issubclass(repo, ABC)

    @pytest.mark.spec("SPEC-EPISODE-005")
    def test_repository_method_naming_convention(self) -> None:
        """メソッド命名規則の確認"""
        # 検索メソッドは find_ または get_ で始まる
        # 永続化メソッドは save, create, update
        # 削除メソッドは delete
        # 確認メソッドは exists
        # アクションメソッドは動詞で始まる(archive, restore, backup等)
        episode_methods = dir(EpisodeRepository)
        project_methods = dir(ProjectRepository)

        # 一般的なメソッドパターンの確認
        search_prefixes = ["find_", "get_"]
        action_verbs = [
            "save",
            "create",
            "update",
            "delete",
            "archive",
            "restore",
            "backup",
            "validate",
            "initialize",
            "bulk_",
            "exists",
        ]

        for method in episode_methods + project_methods:
            if method.startswith("_"):
                continue

            # メソッド名が適切なパターンに従っているか確認
            valid_pattern = False
            for prefix in search_prefixes + action_verbs:
                if method.startswith(prefix):
                    valid_pattern = True
                    break

            # 抽象メソッド以外はパターンに従うべき
            if not method.startswith("__") and method not in ["__class__", "__module__"]:
                assert valid_pattern, f"Method '{method}' does not follow naming convention"



#!/usr/bin/env python3

"""Domain.services.step_selector_service
Where: Domain service module providing reusable business logic.
What: Houses service routines and helpers implemented in this module.
Why: Makes this service behaviour accessible to application workflows.
"""

from __future__ import annotations

"""ステップ選択サービス

SPEC-STEPWISE-WRITING-001に基づく正規表現によるステップ選択機能
B20準拠: Functional Core実装（純粋関数）
"""


import re

from noveler.domain.interfaces.logger_interface import ILogger, NullLogger


class InvalidStepPatternError(ValueError):
    """無効なステップパターンエラー"""

    def __init__(self, pattern: str, reason: str, logger: ILogger | None = None) -> None:
        self._logger = logger or NullLogger()
        self.pattern = pattern
        self.reason = reason
        super().__init__(f"無効なステップパターン '{pattern}': {reason}")


class StepSelectorService:
    """ステップ選択サービス

    正規表現による柔軟なステップ選択機能を提供
    B20準拠: Functional Core（副作用なし）
    """

    # ステップエイリアス定義
    STEP_ALIASES: dict[str, str] = {
        # グループエイリアス
        "structure": "0-9",  # 構造設計フェーズ
        "writing": "10-17",  # 執筆実装フェーズ
        "quality": "5-9,13-14",  # 品質関連
        "optimization": "3,11,12",  # 最適化関連
        # 個別エイリアス（わかりやすい名前）
        "scope": "0",  # スコープ定義
        "story": "1",  # 物語構造
        "phase": "2",  # 段階構造
        "section": "3",  # セクション最適化
        "scene": "4",  # シーン設計
        "logic": "5",  # 論理検証
        "character": "6",  # キャラクター一貫性
        "dialogue": "7",  # 会話設計
        "emotion": "8",  # 感情曲線
        "world": "9",  # 世界観構築
        "sensory": "10",  # 五感描写設計
        "props": "11",  # 小道具・世界観設計
        "manuscript": "12",  # 初稿生成
        "length": "13",  # 文字数最適化
        "readability": "14",  # 文体・可読性パス
        "gate": "15",  # 必須品質ゲート
        "certification": "16",  # DOD & KPI統合品質認定
        "publish": "17",  # 公開準備
    }

    # ステップ番号からサービス名へのマッピング（A38ガイド準拠：0-17の18ステップ）
    STEP_SERVICE_MAPPING: dict[int, str] = {
        0: "ScopeDefinerService",                    # STEP 0: スコープ定義
        1: "StoryStructureDesignerService",         # STEP 1: 大骨（章の目的線）
        2: "PhaseStructureDesignerService",         # STEP 2: 中骨（段階目標・行動フェーズ）
        3: "SectionBalanceOptimizerService",        # STEP 3: セクションバランス設計
        4: "SceneBeatDesignerService",              # STEP 4: 小骨（シーン／ビート）
        5: "LogicValidatorService",                 # STEP 5: 論理検証
        6: "CharacterConsistencyService",          # STEP 6: キャラクター一貫性検証
        7: "DialogueDesignerService",              # STEP 7: 会話設計
        8: "EmotionCurveDesignerService",          # STEP 8: 感情曲線
        9: "SceneSettingService",                  # STEP 9: 情景設計
        10: "SensoryDesignService",                # STEP 10: 五感描写設計
        11: "PropsWorldBuildingService",           # STEP 11: 小道具・世界観設計
        12: "ManuscriptGeneratorService",          # STEP 12: 初稿生成
        13: "TextLengthOptimizerService",          # STEP 13: 文字数最適化
        14: "ReadabilityOptimizerService",         # STEP 14: 文体・可読性パス
        15: "QualityGateService",                  # STEP 15: 必須品質ゲート
        16: "QualityCertificationService",         # STEP 16: DOD & KPI統合品質認定
        17: "PublishingPreparationService",        # STEP 17: 公開準備
    }

    # 正規表現マッチング用サービス名パターン（A38ガイド準拠：18ステップ）
    SERVICE_NAME_PATTERNS: dict[str, list[int]] = {
        r".*optimizer": [3, 13, 14],  # SectionBalanceOptimizer, TextLengthOptimizer, ReadabilityOptimizer
        r".*designer": [
            1,  # StoryStructureDesigner
            2,  # PhaseStructureDesigner
            4,  # SceneBeatDesigner
            7,  # DialogueDesigner
            8,  # EmotionCurveDesigner
            10, # SensoryDesigner
        ],
        r".*service": [
            9,  # SceneSettingService
            11, # PropsWorldBuildingService
            15, # QualityGateService
            16, # QualityCertificationService
            17, # PublishingPreparationService
        ],
        r".*validator": [5],  # LogicValidator
        r".*consistency": [6],  # CharacterConsistency
        r".*generator": [12],  # ManuscriptGenerator
        r".*definer": [0],  # ScopeDefiner
        r".*building": [11],  # PropsWorldBuilding
        r".*curve": [8],  # EmotionCurve
        r".*setting": [9],  # SceneSetting
        r".*sensory": [10],  # SensoryDesign
        r".*gate": [15],  # QualityGate
        r".*certification": [16],  # QualityCertification
        r".*preparation": [17],  # PublishingPreparation
    }

    def __init__(self, logger: ILogger | None = None) -> None:
        """初期化"""
        self._logger = logger or NullLogger()
        self._logger.debug("StepSelectorService initialized")

    def parse_step_pattern(self, pattern: str) -> list[int]:
        """ステップパターンを解析して実行対象ステップリストを生成

        B20準拠: Functional Core（純粋関数）

        Args:
            pattern: ステップパターン文字列

        Returns:
            list[int]: 実行対象ステップリスト（0-17）

        Raises:
            InvalidStepPatternError: 無効なパターンの場合

        Examples:
            >>> service = StepSelectorService()
            >>> service.parse_step_pattern("0-3")
            [0, 1, 2, 3]
            >>> service.parse_step_pattern("structure")
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
            >>> service.parse_step_pattern("scope|story")
            [0, 1]
            >>> service.parse_step_pattern(".*optimizer")
            [3, 13, 14]
        """
        if not pattern or not isinstance(pattern, str):
            raise InvalidStepPatternError(pattern, "パターンが空または無効な型です")

        try:
            self._logger.debug(
                "Parsing step pattern: %s",
                pattern,
            )

            # 除外パターンの処理
            include_steps, exclude_steps = self._split_exclusion_pattern(pattern)

            # 包含ステップの解析
            selected_steps = self._parse_include_pattern(include_steps)

            # 除外ステップの解析と適用
            if exclude_steps:
                excluded_steps = self._parse_exclude_pattern(exclude_steps)
                selected_steps = [step for step in selected_steps if step not in excluded_steps]

            # 重複除去とソート
            result = sorted(set(selected_steps))

            # 結果の妥当性確認
            self._validate_step_list(result)

            self._logger.debug(
                "Pattern '%s' resolved to steps: %s",
                pattern,
                result,
            )
            return result

        except InvalidStepPatternError:
            raise
        except Exception as e:
            raise InvalidStepPatternError(pattern, f"パターン解析エラー: {e!s}")

    def get_service_name(self, step_number: int) -> str:
        """ステップ番号からサービス名を取得

        Args:
            step_number: ステップ番号（0-17）

        Returns:
            str: サービス名

        Raises:
            ValueError: 無効なステップ番号の場合
        """
        if not isinstance(step_number, int) or step_number < 0 or step_number > 17:
            msg = f"無効なステップ番号: {step_number}"
            raise ValueError(msg)

        return self.STEP_SERVICE_MAPPING[step_number]

    def _split_exclusion_pattern(self, pattern: str) -> tuple[str, list[str]]:
        """除外パターンを分離

        Args:
            pattern: 元のパターン文字列

        Returns:
            tuple[包含パターン, 除外パターンリスト]
        """
        parts = pattern.split(",")
        include_parts = []
        exclude_parts = []

        for part in parts:
            part = part.strip()
            if part.startswith("!"):
                exclude_parts.append(part[1:])  # !を除去
            else:
                include_parts.append(part)

        include_pattern = ",".join(include_parts) if include_parts else "0-17"
        return include_pattern, exclude_parts

    def _parse_include_pattern(self, pattern: str, _depth: int = 0) -> list[int]:
        """包含パターンを解析

        Args:
            pattern: 包含パターン文字列
            _depth: 再帰の深度（無限ループ防止）

        Returns:
            list[int]: 選択されたステップリスト
        """
        if _depth > 5:  # 無限ループ防止
            raise InvalidStepPatternError(pattern, "パターンの解析が深すぎます（無限ループの可能性）")

        steps: set[int] = set()

        # カンマ区切りで分割
        parts = [part.strip() for part in pattern.split(",") if part.strip()]

        for part in parts:
            part_steps = self._parse_single_pattern(part, _depth + 1)
            steps.update(part_steps)

        return list(steps)

    def _parse_exclude_pattern(self, exclude_patterns: list[str]) -> set[int]:
        """除外パターンを解析

        Args:
            exclude_patterns: 除外パターンリスト

        Returns:
            set[int]: 除外対象ステップセット
        """
        excluded_steps: set[int] = set()

        for pattern in exclude_patterns:
            pattern_steps = self._parse_single_pattern(pattern, 0)  # 除外パターンは深度0から開始
            excluded_steps.update(pattern_steps)

        return excluded_steps

    def _parse_single_pattern(self, pattern: str, _depth: int = 0) -> list[int]:
        """単一パターンを解析

        Args:
            pattern: 単一パターン文字列
            _depth: 再帰の深度（無限ループ防止）

        Returns:
            list[int]: 解析されたステップリスト
        """
        # OR演算子の処理
        if "|" in pattern:
            return self._parse_or_pattern(pattern, _depth)

        # 範囲指定の処理（例: 0-3）
        if "-" in pattern and re.match(r"^\d+-\d+$", pattern):
            return self._parse_range_pattern(pattern)

        # 数値の処理
        if pattern.isdigit():
            step_num = int(pattern)
            if 0 <= step_num <= 17:
                return [step_num]
            raise InvalidStepPatternError(pattern, f"ステップ番号は0-17の範囲で指定してください: {step_num}")

        # 「all」パターンの処理
        if pattern.lower() == "all":
            return list(range(18))  # 0-17の全ステップ

        # エイリアスの処理
        if pattern in self.STEP_ALIASES:
            alias_pattern = self.STEP_ALIASES[pattern]
            return self._parse_include_pattern(
                alias_pattern, _depth
            )  # カンマ区切り対応のため_parse_include_patternを使用

        # 正規表現パターンの処理
        return self._parse_regex_pattern(pattern)

    def _parse_or_pattern(self, pattern: str, _depth: int = 0) -> list[int]:
        """OR演算子パターンを解析

        Args:
            pattern: OR演算子を含むパターン（例: "scope|story"）
            _depth: 再帰の深度（無限ループ防止）

        Returns:
            list[int]: 解析されたステップリスト
        """
        steps: set[int] = set()

        parts = pattern.split("|")
        for part in parts:
            part = part.strip()
            if part:
                part_steps = self._parse_single_pattern(part, _depth)
                steps.update(part_steps)

        return list(steps)

    def _parse_range_pattern(self, pattern: str) -> list[int]:
        """範囲パターンを解析

        Args:
            pattern: 範囲パターン（例: "0-3"）

        Returns:
            list[int]: 範囲内のステップリスト
        """
        try:
            start_str, end_str = pattern.split("-")
            start = int(start_str)
            end = int(end_str)

            if start < 0 or end < 0:
                raise InvalidStepPatternError(pattern, "負の数は指定できません")

            if start > end:
                raise InvalidStepPatternError(pattern, f"開始値({start})が終了値({end})より大きいです")

            if end > 17:
                raise InvalidStepPatternError(pattern, f"ステップ番号は17以下で指定してください: {end}")

            return list(range(start, end + 1))

        except ValueError as e:
            raise InvalidStepPatternError(pattern, f"範囲パターンの解析に失敗: {e!s}")

    def _parse_regex_pattern(self, pattern: str) -> list[int]:
        """正規表現パターンを解析

        Args:
            pattern: 正規表現パターン（例: ".*optimizer"）

        Returns:
            list[int]: マッチするステップリスト
        """
        try:
            compiled_regex = re.compile(pattern, re.IGNORECASE)
            matched_steps: set[int] = set()

            # サービス名パターンとのマッチング
            for service_pattern, steps in self.SERVICE_NAME_PATTERNS.items():
                if compiled_regex.match(service_pattern):
                    matched_steps.update(steps)

            # 個別エイリアスとのマッチング
            for alias, alias_pattern in self.STEP_ALIASES.items():
                if compiled_regex.match(alias):
                    alias_steps = self._parse_include_pattern(alias_pattern, 0)  # カンマ区切り対応、深度0から開始
                    matched_steps.update(alias_steps)

            if not matched_steps:
                # 直接的なサービス名マッチングを試行
                for step_num, service_name in self.STEP_SERVICE_MAPPING.items():
                    if compiled_regex.search(service_name.lower()):
                        matched_steps.add(step_num)

            if not matched_steps:
                raise InvalidStepPatternError(pattern, "正規表現にマッチするステップが見つかりません")

            return list(matched_steps)

        except re.error as e:
            raise InvalidStepPatternError(pattern, f"無効な正規表現: {e!s}")

    def _validate_step_list(self, steps: list[int]) -> None:
        """ステップリストの妥当性確認

        Args:
            steps: ステップリスト

        Raises:
            InvalidStepPatternError: 無効なステップが含まれる場合
        """
        if not steps:
            msg = ""
            raise InvalidStepPatternError(msg, "実行対象ステップが選択されていません")

        for step in steps:
            if not isinstance(step, int) or step < 0 or step > 17:
                raise InvalidStepPatternError(str(step), f"無効なステップ番号: {step}")

    def get_pattern_examples(self) -> dict[str, list[int]]:
        """パターン例とその結果を取得

        Returns:
            dict[str, list[int]]: パターン例辞書
        """
        return {
            # 基本パターン
            "0": [0],
            "17": [17],
            "0-3": [0, 1, 2, 3],
            "10-12": [10, 11, 12],
            "all": list(range(18)),  # 全ステップ（0-17）
            # エイリアス
            "scope": [0],
            "structure": list(range(10)),
            "writing": list(range(10, 18)),
            "quality": [5, 6, 7, 8, 9, 15, 16],
            # OR演算子
            "scope|story": [0, 1],
            "manuscript|gate": [12, 15],
            # カンマ区切り
            "0,5,10": [0, 5, 10],
            "0-2,12-14": [0, 1, 2, 12, 13, 14],
            # 正規表現
            ".*optimizer": [3, 13, 14],
            ".*designer": [1, 2, 4, 7, 8, 10],
            ".*service": [9, 11, 15, 16, 17],
            # 除外パターン
            "0-5,!3": [0, 1, 2, 4, 5],
            "structure,!quality": [0, 1, 2, 3, 4],
        }

    def validate_pattern(self, pattern: str) -> bool:
        """パターンの妥当性を検証

        Args:
            pattern: 検証対象パターン

        Returns:
            bool: パターンが有効かどうか
        """
        try:
            self.parse_step_pattern(pattern)
            return True
        except InvalidStepPatternError:
            return False


def create_step_selector() -> StepSelectorService:
    """StepSelectorServiceのファクトリ関数

    Returns:
        StepSelectorService: 初期化済みのインスタンス
    """
    return StepSelectorService()



#!/usr/bin/env python3
# File: tests/unit/infrastructure/test_yaml_quality_record_repository.py
# Purpose: Verify YAML-based quality repositories manage persistence paths,
#          backups, and transaction orchestration correctly.
# Context: Unit test suite covering infrastructure repositories used by MCP
#          and CLI flows. Exercises path resolution via PathService adapters.
"""Unit tests for YAML-backed quality repositories and transaction helpers."""

from datetime import timedelta
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import Mock, patch

import pytest
import yaml

from noveler.domain.entities.quality_record import QualityRecord
from noveler.domain.exceptions import QualityRecordError
from noveler.domain.value_objects.project_time import project_now
from noveler.domain.value_objects.quality_check_result import (
    AutoFix,
    CategoryScores,
    QualityCheckResult,
    QualityError,
    QualityScore,
)
from noveler.infrastructure.adapters.path_service_adapter import create_path_service
from noveler.infrastructure.repositories.yaml_quality_record_repository import (
    YamlEpisodeManagementRepository,
    YamlQualityRecordRepository,
    YamlRecordTransactionManager,
    YamlRevisionHistoryRepository,
)


class TestYamlQualityRecordRepository:
    """YAML品質記録リポジトリのテスト"""

    def setup_method(self) -> None:
        """テストセットアップ"""
        self.temp_dir = TemporaryDirectory()
        self.test_dir = Path(self.temp_dir.name)
        self.repository = YamlQualityRecordRepository(self.test_dir)

    def teardown_method(self) -> None:
        """テストクリーンアップ"""
        self.temp_dir.cleanup()

    def create_sample_quality_record(self, project_name: str = "test_project") -> QualityRecord:
        """テスト用品質記録作成"""
        record = QualityRecord(project_name)

        result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[QualityError(type="punctuation", message="記号エラー", line_number=10)],
            warnings=[],
            auto_fixes=[AutoFix(type="punctuation_fix", description="記号修正", count=1)],
        )

        record.add_quality_check_result(result)
        return record

    def test_find_by_project_file_not_exists(self) -> None:
        """存在しないプロジェクトファイルの場合はNoneを返す"""
        result = self.repository.find_by_project("nonexistent_project")
        assert result is None

    def test_save_new_quality_record(self) -> None:
        """新規品質記録の保存"""
        record = self.create_sample_quality_record()

        # 保存実行
        self.repository.save(record)

        # ファイルが作成されたことを確認
        path_service = create_path_service(self.test_dir / "test_project")
        file_path = path_service.get_quality_record_file()
        assert file_path.exists()

        # ファイル内容の確認
        with Path(file_path).open(encoding="utf-8") as f:
            data = yaml.safe_load(f)

        assert data["metadata"]["project_name"] == "test_project"
        assert data["metadata"]["entry_count"] == 1
        assert len(data["quality_checks"]) == 1
        assert data["quality_checks"][0]["episode_number"] == 1

    def test_find_by_project_existing_file(self) -> None:
        """既存ファイルから品質記録を読み込み"""
        # まず保存
        original_record = self.create_sample_quality_record()
        self.repository.save(original_record)

        # 読み込み
        loaded_record = self.repository.find_by_project("test_project")

        assert loaded_record is not None
        assert loaded_record.project_name == "test_project"
        assert loaded_record.entry_count == 1

        # エントリ内容の確認
        latest_entry = loaded_record.get_latest_for_episode(1)
        assert latest_entry is not None
        assert latest_entry.quality_result.episode_number == 1

    def test_save_update_existing_record(self) -> None:
        """既存記録の更新"""
        # 初回保存
        record = self.create_sample_quality_record()
        self.repository.save(record)

        # 新しい結果を追加
        new_result = QualityCheckResult(
            episode_number=2,
            timestamp=project_now().datetime + timedelta(minutes=1),
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(90.0),
                composition=QualityScore.from_float(85.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(87.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        record.add_quality_check_result(new_result)

        # 更新保存
        self.repository.save(record)

        # 再読み込みして確認
        loaded_record = self.repository.find_by_project("test_project")
        assert loaded_record.entry_count == 2
        assert loaded_record.get_latest_for_episode(2) is not None

    def test_file_permission_error_handling(self) -> None:
        """ファイル権限エラーの処理"""
        record = self.create_sample_quality_record()

        # ディレクトリを読み取り専用に設定(Windows環境での権限制御をシミュレート)
        with patch("builtins.open", side_effect=PermissionError("Permission denied")):
            with pytest.raises(QualityRecordError, match="Failed to save quality record"):
                self.repository.save(record)

    def test_yaml_parsing_error_handling(self) -> None:
        """不正なYAMLファイルの処理"""
        # 不正なYAMLファイルを作成
        path_service = create_path_service(self.test_dir / "test_project")
        file_path = path_service.get_quality_record_file()
        file_path.parent.mkdir(parents=True, exist_ok=True)

        with Path(file_path).open("w", encoding="utf-8") as f:
            f.write("invalid: yaml: content: [unclosed")

        # パースエラーが適切に処理されることを確認
        with pytest.raises(QualityRecordError, match="Failed to parse quality record file"):
            self.repository.find_by_project("test_project")

    def test_backup_creation_on_save(self) -> None:
        """保存時のバックアップ作成"""
        record = self.create_sample_quality_record()

        # 初回保存
        self.repository.save(record)

        # ファイルを変更して再保存
        new_result = QualityCheckResult(
            episode_number=2,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(90.0),
                composition=QualityScore.from_float(85.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(87.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        record.add_quality_check_result(new_result)

        # 再保存(バックアップが作成される)
        self.repository.save(record)

        # バックアップファイルの存在確認
        path_service = create_path_service(self.test_dir / "test_project")
        backup_dir = path_service.get_management_dir() / ".backup"
        backup_files = list(backup_dir.glob("品質記録_*.yaml"))
        assert len(backup_files) >= 1


class TestYamlEpisodeManagementRepository:
    """YAML話数管理リポジトリのテスト"""

    def setup_method(self) -> None:
        """テストセットアップ"""
        self.temp_dir = TemporaryDirectory()
        self.test_dir = Path(self.temp_dir.name)
        self.repository = YamlEpisodeManagementRepository()

    def teardown_method(self) -> None:
        """テストクリーンアップ"""
        self.temp_dir.cleanup()

    def test_update_episode_status_new_episode(self) -> None:
        """新規エピソードの状態更新"""
        project_path = self.test_dir / "test_project"
        project_path.mkdir(parents=True, exist_ok=True)
        path_service = create_path_service(project_path)
        episode_file = path_service.get_episode_management_file()
        episode_file.parent.mkdir(parents=True, exist_ok=True)

        # 初期ファイル作成
        initial_data = {
            "episodes": [],
            "metadata": {"last_updated": project_now().datetime.isoformat(), "total_episodes": 0},
        }

        with Path(episode_file).open("w", encoding="utf-8") as f:
            yaml.dump(initial_data, f, default_flow_style=False, allow_unicode=True)

        # 品質チェック結果
        quality_result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        # 更新実行
        self.repository.update_quality_scores(project_path, 1, quality_result)

        # 結果確認
        with Path(episode_file).open(encoding="utf-8") as f:
            data = yaml.safe_load(f)

        assert len(data["episodes"]) == 1
        assert data["episodes"][0]["episode_number"] == 1
        assert data["episodes"][0]["quality_score"] == 83.75  # 平均
        assert data["metadata"]["total_episodes"] == 1

    def test_update_episode_status_existing_episode(self) -> None:
        """既存エピソードの状態更新"""
        project_path = self.test_dir / "test_project"
        project_path.mkdir(parents=True, exist_ok=True)
        path_service = create_path_service(project_path)
        episode_file = path_service.get_episode_management_file()
        episode_file.parent.mkdir(parents=True, exist_ok=True)

        # 既存データ
        existing_data = {
            "episodes": [
                {
                    "episode_number": 1,
                    "title": "テストエピソード",
                    "quality_score": 80.0,
                    "last_check": (project_now().datetime - timedelta(days=1)).isoformat(),
                    "status": "draft",
                }
            ],
            "metadata": {"last_updated": (project_now().datetime - timedelta(days=1)).isoformat(), "total_episodes": 1},
        }

        with Path(episode_file).open("w", encoding="utf-8") as f:
            yaml.dump(existing_data, f, default_flow_style=False, allow_unicode=True)

        # 新しい品質チェック結果
        quality_result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(90.0),
                composition=QualityScore.from_float(85.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(87.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        # 更新実行
        self.repository.update_quality_scores(project_path, 1, quality_result)

        # 結果確認
        with Path(episode_file).open(encoding="utf-8") as f:
            data = yaml.safe_load(f)

        assert len(data["episodes"]) == 1
        assert data["episodes"][0]["quality_score"] == 87.5  # 新しい平均
        assert data["metadata"]["total_episodes"] == 1


class TestYamlRevisionHistoryRepository:
    """YAML改訂履歴リポジトリのテスト"""

    def setup_method(self) -> None:
        """テストセットアップ"""
        self.temp_dir = TemporaryDirectory()
        self.test_dir = Path(self.temp_dir.name)
        self.repository = YamlRevisionHistoryRepository()

    def teardown_method(self) -> None:
        """テストクリーンアップ"""
        self.temp_dir.cleanup()

    def test_add_quality_check_revision_new_file(self) -> None:
        """新規改訂履歴の追加"""
        project_path = self.test_dir / "test_project"
        project_path.mkdir(parents=True, exist_ok=True)
        path_service = create_path_service(project_path)
        revision_file = path_service.get_revision_history_file()

        quality_result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[QualityError(type="punctuation", message="記号エラー", line_number=10)],
            warnings=[],
            auto_fixes=[AutoFix(type="punctuation_fix", description="記号修正", count=1)],
        )

        # 改訂履歴追加
        self.repository.add_quality_revision(project_path, quality_result)

        # 結果確認
        assert revision_file.exists()

        with Path(revision_file).open(encoding="utf-8") as f:
            data = yaml.safe_load(f)

        assert len(data["revisions"]) == 1
        revision = data["revisions"][0]
        assert revision["episode_number"] == 1
        assert revision["revision_type"] == "quality_check"
        assert revision["quality_score"] == 83.75
        assert revision["error_count"] == 1
        assert revision["auto_fix_count"] == 1

    def test_add_quality_check_revision_existing_file(self) -> None:
        """既存改訂履歴への追加"""
        project_path = self.test_dir / "test_project"
        project_path.mkdir(parents=True, exist_ok=True)
        path_service = create_path_service(project_path)
        revision_file = path_service.get_revision_history_file()
        revision_file.parent.mkdir(parents=True, exist_ok=True)

        # 既存データ
        existing_data = {
            "revisions": [
                {
                    "id": "rev-001",
                    "timestamp": (project_now().datetime - timedelta(days=1)).isoformat(),
                    "episode_number": 1,
                    "revision_type": "manual_edit",
                    "description": "手動編集",
                }
            ],
            "metadata": {
                "total_revisions": 1,
                "last_updated": (project_now().datetime - timedelta(days=1)).isoformat(),
            },
        }

        with Path(revision_file).open("w", encoding="utf-8") as f:
            yaml.dump(existing_data, f, default_flow_style=False, allow_unicode=True)

        quality_result = QualityCheckResult(
            episode_number=2,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(90.0),
                composition=QualityScore.from_float(85.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(87.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        # 新しい改訂履歴追加
        self.repository.add_quality_revision(project_path, quality_result)

        # 結果確認
        with Path(revision_file).open(encoding="utf-8") as f:
            data = yaml.safe_load(f)

        assert len(data["revisions"]) == 2
        assert data["metadata"]["total_revisions"] == 2

        # 新しい改訂履歴が先頭に追加されることを確認
        latest_revision = data["revisions"][0]
        assert latest_revision["episode_number"] == 2
        assert latest_revision["revision_type"] == "quality_check"


class TestYamlRecordTransactionManager:
    """YAML記録トランザクションマネージャーのテスト"""

    def setup_method(self) -> None:
        """テストセットアップ"""
        self.temp_dir = TemporaryDirectory()
        self.test_dir = Path(self.temp_dir.name)

        self.quality_repo = Mock()
        self.episode_repo = Mock()
        self.revision_repo = Mock()

        self.transaction_manager = YamlRecordTransactionManager(
            quality_record_repository=self.quality_repo,
            episode_management_repository=self.episode_repo,
            revision_history_repository=self.revision_repo,
        )

    def teardown_method(self) -> None:
        """テストクリーンアップ"""
        self.temp_dir.cleanup()

    def test_successful_transaction(self) -> None:
        """正常なトランザクション実行"""
        record = QualityRecord("test_project")
        quality_result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        project_path = self.test_dir / "test_project"

        # トランザクション実行
        with self.transaction_manager.begin_transaction() as transaction:
            transaction.update_quality_record(record)
            transaction.update_episode_management(project_path, 1, quality_result)
            transaction.update_revision_history(project_path, quality_result)
            transaction.commit()

        # リポジトリメソッドが呼ばれたことを確認
        self.quality_repo.save.assert_called_once_with(record)
        self.episode_repo.update_quality_scores.assert_called_once_with(project_path, 1, quality_result)
        self.revision_repo.add_quality_revision.assert_called_once_with(project_path, quality_result)

    def test_transaction_rollback_on_error(self) -> None:
        """エラー時のトランザクションロールバック"""
        record = QualityRecord("test_project")
        quality_result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        project_path = self.test_dir / "test_project"

        # episode_repoでエラーを発生させる
        self.episode_repo.update_quality_scores.side_effect = Exception("Episode update failed")

        # トランザクション実行(エラーが発生)
        with pytest.raises(Exception, match="Episode update failed"):
            with self.transaction_manager.begin_transaction() as transaction:
                transaction.update_quality_record(record)
                transaction.update_episode_management(project_path, 1, quality_result)
                transaction.update_revision_history(project_path, quality_result)
                transaction.commit()

        # ロールバックにより、他の操作も実行されないことを確認
        self.revision_repo.add_quality_revision.assert_not_called()

    def test_transaction_context_manager(self) -> None:
        """トランザクションのコンテキストマネージャー機能"""
        record = QualityRecord("test_project")

        # 正常終了の場合
        with self.transaction_manager.begin_transaction() as transaction:
            transaction.update_quality_record(record)
            # コミットを明示的に呼ばない場合、自動コミット

        self.quality_repo.save.assert_called_once()

    def test_transaction_explicit_rollback(self) -> None:
        """明示的なロールバック"""
        record = QualityRecord("test_project")

        with self.transaction_manager.begin_transaction() as transaction:
            transaction.update_quality_record(record)
            # 何らかの理由でロールバック
            transaction.rollback()

        # ロールバック後は実際の保存操作は行われない
        self.quality_repo.save.assert_not_called()



# File: tests/unit/infrastructure/json/mcp/test_performance_monitor.py
# Purpose: Unit tests for PerformanceMonitor class
# Context: Validates performance metrics collection and aggregation

"""Comprehensive unit tests for PerformanceMonitor monitoring system.

Tests cover:
- Execution metrics recording
- Cache metrics tracking
- Performance aggregation
- Optimization suggestions
- History management
- Async monitoring
"""

import asyncio
from unittest.mock import Mock

import pytest

from noveler.infrastructure.json.mcp.servers.performance_monitor import (
    CacheMetrics,
    ExecutionMetrics,
    PerformanceMonitor,
)
from noveler.infrastructure.logging.unified_logger import get_logger


@pytest.fixture
def mock_logger():
    """Create a mock logger for testing."""
    return Mock()


@pytest.fixture
def performance_monitor(mock_logger):
    """Create a PerformanceMonitor instance for testing."""
    return PerformanceMonitor(logger=mock_logger, history_size=10)


class TestExecutionMetrics:
    """Test ExecutionMetrics dataclass."""

    def test_execution_metrics_success(self):
        """Test ExecutionMetrics with successful execution."""
        metric = ExecutionMetrics(
            tool_name="test_tool",
            start_time=1000.0,
            end_time=1100.0,
            duration_ms=100.0,
            status="success",
        )
        assert metric.success is True

    def test_execution_metrics_failure(self):
        """Test ExecutionMetrics with failed execution."""
        metric = ExecutionMetrics(
            tool_name="test_tool",
            start_time=1000.0,
            end_time=1100.0,
            duration_ms=100.0,
            status="failure",
            error="Test error",
        )
        assert metric.success is False
        assert metric.error == "Test error"


class TestCacheMetrics:
    """Test CacheMetrics dataclass."""

    def test_cache_metrics_hit_rate_calculation(self):
        """Test cache hit rate calculation."""
        cache = CacheMetrics(cache_name="TestCache", hits=80, misses=20)
        assert cache.hit_rate == 80.0

    def test_cache_metrics_hit_rate_zero(self):
        """Test cache hit rate when no operations occurred."""
        cache = CacheMetrics(cache_name="TestCache")
        assert cache.hit_rate == 0.0

    def test_cache_metrics_hit_rate_all_misses(self):
        """Test cache hit rate with all misses."""
        cache = CacheMetrics(cache_name="TestCache", hits=0, misses=100)
        assert cache.hit_rate == 0.0


class TestPerformanceMonitorInitialization:
    """Test PerformanceMonitor initialization."""

    def test_monitor_initializes_with_logger(self, mock_logger):
        """Test that monitor initializes with logger."""
        monitor = PerformanceMonitor(logger=mock_logger)
        assert monitor._logger is mock_logger

    def test_monitor_initializes_metrics(self, performance_monitor):
        """Test that metrics are initialized."""
        assert performance_monitor._total_executions == 0
        assert performance_monitor._successful_executions == 0
        assert performance_monitor._failed_executions == 0
        assert len(performance_monitor._cache_metrics) == 3


class TestExecutionMetricsRecording:
    """Test recording of execution metrics."""

    def test_record_successful_execution(self, performance_monitor):
        """Test recording a successful execution."""
        performance_monitor.record_execution(
            tool_name="test_tool",
            duration_ms=100.0,
            status="success",
        )

        assert performance_monitor._total_executions == 1
        assert performance_monitor._successful_executions == 1
        assert performance_monitor._failed_executions == 0

    def test_record_failed_execution(self, performance_monitor):
        """Test recording a failed execution."""
        performance_monitor.record_execution(
            tool_name="test_tool",
            duration_ms=100.0,
            status="failure",
            error="Test error",
        )

        assert performance_monitor._total_executions == 1
        assert performance_monitor._successful_executions == 0
        assert performance_monitor._failed_executions == 1

    def test_record_multiple_executions(self, performance_monitor):
        """Test recording multiple executions."""
        for i in range(5):
            performance_monitor.record_execution(
                tool_name=f"tool_{i}",
                duration_ms=100.0 + i * 10,
                status="success" if i % 2 == 0 else "failure",
            )

        assert performance_monitor._total_executions == 5
        assert performance_monitor._successful_executions == 3
        assert performance_monitor._failed_executions == 2

    def test_execution_history_pruning(self, performance_monitor):
        """Test that execution history is pruned when exceeding size."""
        # Record more than history_size (10) executions
        for i in range(15):
            performance_monitor.record_execution(
                tool_name="test_tool",
                duration_ms=100.0,
                status="success",
            )

        # History should be limited to history_size
        assert len(performance_monitor._execution_history) == 10


class TestCacheMetricsRecording:
    """Test cache metrics recording."""

    def test_record_cache_hit(self, performance_monitor):
        """Test recording a cache hit."""
        performance_monitor.record_cache_hit("FileIOCache")

        assert performance_monitor._cache_metrics["FileIOCache"].hits == 1
        assert performance_monitor._cache_metrics["FileIOCache"].misses == 0

    def test_record_cache_miss(self, performance_monitor):
        """Test recording a cache miss."""
        performance_monitor.record_cache_miss("FileIOCache")

        assert performance_monitor._cache_metrics["FileIOCache"].hits == 0
        assert performance_monitor._cache_metrics["FileIOCache"].misses == 1

    def test_record_cache_eviction(self, performance_monitor):
        """Test recording cache evictions."""
        performance_monitor.record_cache_eviction("JSONCache", count=5)

        assert performance_monitor._cache_metrics["JSONCache"].evictions == 5

    def test_update_cache_size(self, performance_monitor):
        """Test updating cache size."""
        performance_monitor.update_cache_size("PathCache", size=42)

        assert performance_monitor._cache_metrics["PathCache"].total_entries == 42


class TestMetricsAggregation:
    """Test metrics aggregation and reporting."""

    def test_get_metrics_empty(self, performance_monitor):
        """Test getting metrics when no executions recorded."""
        metrics = performance_monitor.get_metrics()

        assert metrics.total_executions == 0
        assert metrics.success_rate == 0.0
        assert metrics.average_execution_time_ms == 0.0

    def test_get_metrics_with_executions(self, performance_monitor):
        """Test getting metrics after recording executions."""
        performance_monitor.record_execution("tool1", 100.0, "success")
        performance_monitor.record_execution("tool2", 200.0, "failure")

        metrics = performance_monitor.get_metrics()

        assert metrics.total_executions == 2
        assert metrics.successful_executions == 1
        assert metrics.failed_executions == 1
        assert metrics.success_rate == 50.0
        assert metrics.average_execution_time_ms == 150.0

    def test_get_metrics_includes_suggestions(self, performance_monitor):
        """Test that metrics include optimization suggestions."""
        # Record slow executions
        for _ in range(10):
            performance_monitor.record_execution("slow_tool", 5000.0, "failure")

        metrics = performance_monitor.get_metrics()

        assert len(metrics.optimization_suggestions) > 0


class TestOptimizationSuggestions:
    """Test optimization suggestion generation."""

    def test_slow_execution_suggestion(self, performance_monitor):
        """Test suggestion for slow executions."""
        # Record very slow execution
        performance_monitor.record_execution("slow_tool", 2000.0, "success")

        metrics = performance_monitor.get_metrics()

        assert any("平均実行時間が長い" in s for s in metrics.optimization_suggestions)

    def test_low_success_rate_suggestion(self, performance_monitor):
        """Test suggestion for low success rate."""
        # Record mostly failed executions
        for _ in range(10):
            performance_monitor.record_execution("unreliable", 100.0, "failure")

        metrics = performance_monitor.get_metrics()

        assert any("成功率が低い" in s for s in metrics.optimization_suggestions)

    def test_cache_eviction_suggestion(self, performance_monitor):
        """Test suggestion for high eviction rate."""
        # Record high evictions
        performance_monitor.update_cache_size("FileIOCache", 10)
        performance_monitor.record_cache_eviction("FileIOCache", count=150)

        metrics = performance_monitor.get_metrics()

        assert any("削除が多い" in s for s in metrics.optimization_suggestions)


class TestAsyncMonitoring:
    """Test async monitoring functionality."""

    @pytest.mark.asyncio
    async def test_start_monitoring_can_be_cancelled(self, performance_monitor):
        """Test that monitoring can be started and cancelled."""
        monitor_task = asyncio.create_task(performance_monitor.start_monitoring())

        # Let it run for a moment
        await asyncio.sleep(0.1)

        # Cancel monitoring
        monitor_task.cancel()

        try:
            await monitor_task
        except asyncio.CancelledError:
            pass  # Expected

        # Should complete without error


class TestPerformanceSummary:
    """Test performance summary generation."""

    def test_get_summary_empty(self, performance_monitor):
        """Test summary when no metrics recorded."""
        summary = performance_monitor.get_summary()

        assert summary["total_executions"] == 0
        assert summary["success_rate_percent"] == 0.0
        assert summary["average_execution_time_ms"] == 0.0

    def test_get_summary_with_metrics(self, performance_monitor):
        """Test summary with recorded metrics."""
        performance_monitor.record_execution("tool1", 150.0, "success")
        performance_monitor.record_cache_hit("FileIOCache")
        performance_monitor.record_cache_miss("JSONCache")

        summary = performance_monitor.get_summary()

        assert summary["total_executions"] == 1
        assert summary["success_rate_percent"] == 100.0
        assert summary["average_execution_time_ms"] == 150.0
        assert "FileIOCache" in summary["cache_hit_rates"]


class TestPerformanceMonitorIntegration:
    """Integration tests with real logger."""

    def test_performance_monitor_with_real_logger(self):
        """Test PerformanceMonitor with real logger."""
        logger = get_logger(__name__)
        monitor = PerformanceMonitor(logger=logger)

        monitor.record_execution("test_tool", 100.0, "success")
        monitor.record_cache_hit("FileIOCache")

        metrics = monitor.get_metrics()

        assert metrics.total_executions == 1
        assert metrics.cache_metrics["FileIOCache"].hits == 1

    def test_performance_monitor_full_workflow(self):
        """Test complete performance monitoring workflow."""
        logger = get_logger(__name__)
        monitor = PerformanceMonitor(logger=logger, history_size=50)

        # Simulate tool execution sequence
        for i in range(10):
            monitor.record_execution(
                tool_name=f"tool_{i % 3}",
                duration_ms=50.0 + i * 10,
                status="success" if i % 3 != 0 else "failure",
            )

            # Simulate cache operations
            if i % 2 == 0:
                monitor.record_cache_hit("FileIOCache")
            else:
                monitor.record_cache_miss("JSONCache")

        # Update cache sizes
        monitor.update_cache_size("FileIOCache", 250)
        monitor.update_cache_size("JSONCache", 95)

        # Get metrics
        metrics = monitor.get_metrics()
        summary = monitor.get_summary()

        assert metrics.total_executions == 10
        assert "timestamp" in summary
        assert "cache_hit_rates" in summary



#!/usr/bin/env python3
"""
Quality Reporting Service - Pure Domain Service

Responsible for generating quality reports, suggestions, and result formatting.
"""

from typing import Any

from noveler.domain.value_objects.project_time import project_now
from noveler.domain.value_objects.quality_types import (
    QualityCheckResult,
    QualityCheckType,
    QualityIssue,
    QualityScore,
    QualitySeverity,
)


class QualityReportingService:
    """
    Domain service for quality reporting and result management

    Responsibilities:
    - Generate comprehensive quality reports
    - Create improvement suggestions
    - Format results for different audiences
    - Calculate overall quality grades
    - Manage quality result persistence
    """

    def __init__(self) -> None:
        """Initialize quality reporting service"""
        self._grade_thresholds = self._load_grade_thresholds()
        self._suggestion_templates = self._load_suggestion_templates()

    def generate_quality_report(
        self,
        scores: list[QualityScore],
        check_type: QualityCheckType,
        project_name: str,
        episode_number: int | None = None,
        execution_time: float | None = None,
    ) -> QualityCheckResult:
        """Generate comprehensive quality report

        Args:
            scores: List of quality scores
            check_type: Type of quality check performed
            project_name: Project identifier
            episode_number: Episode number
            execution_time: Check execution time

        Returns:
            QualityCheckResult: Complete quality check result
        """
        # Calculate totals
        total_score = sum(score.score for score in scores)
        max_total_score = sum(score.max_score for score in scores)
        overall_percentage = (total_score / max_total_score * 100) if max_total_score > 0 else 0

        # Collect all issues
        all_issues = []
        for score in scores:
            all_issues.extend(score.issues)

        # Generate grade
        grade = self.determine_quality_grade(overall_percentage)

        # Generate suggestions
        suggestions = self.generate_improvement_suggestions(all_issues, check_type, overall_percentage)

        return QualityCheckResult(
            success=True,
            check_type=check_type,
            total_score=total_score,
            max_total_score=max_total_score,
            overall_percentage=overall_percentage,
            grade=grade,
            scores=scores,
            all_issues=all_issues,
            suggestions=suggestions,
            execution_time=execution_time,
            check_timestamp=project_now().datetime,
        )

    def determine_quality_grade(self, percentage: float) -> str:
        """Determine quality grade based on score percentage

        Args:
            percentage: Overall quality percentage

        Returns:
            str: Quality grade (A+, A, B+, etc.)
        """
        for grade, threshold in self._grade_thresholds.items():
            if percentage >= threshold:
                return grade

        return "F"  # Fallback grade

    def generate_improvement_suggestions(
        self, issues: list[QualityIssue], check_type: QualityCheckType, overall_percentage: float
    ) -> list[str]:
        """Generate improvement suggestions based on issues and context

        Args:
            issues: List of quality issues
            check_type: Type of quality check
            overall_percentage: Overall quality percentage

        Returns:
            list[str]: Generated improvement suggestions
        """
        suggestions = []

        # Priority-based suggestions
        critical_issues = [issue for issue in issues if issue.severity == QualitySeverity.CRITICAL]
        error_issues = [issue for issue in issues if issue.severity == QualitySeverity.ERROR]
        warning_issues = [issue for issue in issues if issue.severity == QualitySeverity.WARNING]

        # Critical issues first
        if critical_issues:
            suggestions.append("Critical issues require immediate attention")
            for issue in critical_issues[:3]:  # Top 3 critical issues
                if issue.suggestion:
                    suggestions.append(f"CRITICAL: {issue.suggestion}")

        # Error issues
        if error_issues:
            suggestions.append("Error-level issues need correction")
            for issue in error_issues[:3]:  # Top 3 errors
                if issue.suggestion:
                    suggestions.append(f"ERROR: {issue.suggestion}")

        # Warning issues (if not too many higher priority issues)
        if len(suggestions) < 7 and warning_issues:
            for issue in warning_issues[:2]:  # Top 2 warnings
                if issue.suggestion:
                    suggestions.append(f"WARNING: {issue.suggestion}")

        # Check-type specific suggestions
        type_suggestions = self._generate_type_specific_suggestions(check_type, overall_percentage)
        suggestions.extend(type_suggestions)

        # Overall quality suggestions
        if overall_percentage < 60:
            suggestions.extend(self._suggestion_templates["low_quality"])
        elif overall_percentage < 80:
            suggestions.extend(self._suggestion_templates["medium_quality"])
        else:
            suggestions.extend(self._suggestion_templates["high_quality"])

        return suggestions[:10]  # Limit to 10 suggestions

    def format_quality_summary(self, result: QualityCheckResult) -> str:
        """Format quality result as human-readable summary

        Args:
            result: Quality check result

        Returns:
            str: Formatted summary
        """
        summary_lines = []

        # Header
        summary_lines.append(f"Quality Check Summary - {result.check_type.value.title()}")
        summary_lines.append("=" * 50)

        # Overall results
        summary_lines.append(f"Overall Score: {result.total_score:.1f}/{result.max_total_score:.1f}")
        summary_lines.append(f"Percentage: {result.overall_percentage:.1f}%")
        summary_lines.append(f"Grade: {result.grade}")

        if result.execution_time:
            summary_lines.append(f"Execution Time: {result.execution_time:.3f}s")

        # Category breakdown
        if result.scores:
            summary_lines.append("\nCategory Breakdown:")
            summary_lines.append("-" * 30)
            for score in result.scores:
                summary_lines.append(
                    f"{score.category}: {score.score:.1f}/{score.max_score:.1f} ({score.percentage:.1f}%)"
                )

        # Issues summary
        if result.all_issues:
            issue_counts = self._count_issues_by_severity(result.all_issues)
            summary_lines.append("\nIssues Found:")
            summary_lines.append("-" * 20)
            for severity, count in issue_counts.items():
                if count > 0:
                    summary_lines.append(f"{severity.value.title()}: {count}")

        # Top suggestions
        if result.suggestions:
            summary_lines.append("\nTop Recommendations:")
            summary_lines.append("-" * 25)
            for i, suggestion in enumerate(result.suggestions[:5], 1):
                summary_lines.append(f"{i}. {suggestion}")

        return "\n".join(summary_lines)

    def create_detailed_report(self, result: QualityCheckResult) -> dict[str, Any]:
        """Create detailed quality report data structure

        Args:
            result: Quality check result

        Returns:
            dict[str, Any]: Detailed report data
        """
        return {
            "summary": {
                "check_type": result.check_type.value,
                "total_score": result.total_score,
                "max_total_score": result.max_total_score,
                "overall_percentage": result.overall_percentage,
                "grade": result.grade,
                "execution_time": result.execution_time,
                "timestamp": result.check_timestamp.isoformat() if result.check_timestamp else None,
            },
            "category_scores": [
                {
                    "category": score.category,
                    "score": score.score,
                    "max_score": score.max_score,
                    "percentage": score.percentage,
                    "issue_count": len(score.issues),
                }
                for score in result.scores
            ],
            "issues": {
                "total_count": len(result.all_issues),
                "by_severity": self._count_issues_by_severity(result.all_issues),
                "by_type": self._count_issues_by_type(result.all_issues),
                "detailed_issues": [
                    {
                        "type": issue.type,
                        "severity": issue.severity.value,
                        "message": issue.message,
                        "suggestion": issue.suggestion,
                        "location": issue.location,
                    }
                    for issue in result.all_issues
                ],
            },
            "suggestions": {"count": len(result.suggestions), "recommendations": result.suggestions},
        }

    def generate_a31_specific_suggestions(self, issues: list[QualityIssue]) -> list[str]:
        """Generate A31 manuscript checklist specific suggestions

        Args:
            issues: List of quality issues

        Returns:
            list[str]: A31-specific suggestions
        """
        suggestions = [
            "A31 manuscript execution checklist evaluation completed",
            "Detailed feedback available in individual item assessments",
        ]

        # Critical A31 issues
        critical_issues = [issue for issue in issues if issue.severity == QualitySeverity.CRITICAL]
        if critical_issues:
            suggestions.append("URGENT: Critical A31 checklist items require immediate attention")
            for issue in critical_issues[:3]:
                if issue.suggestion:
                    suggestions.append(f"Critical: {issue.suggestion}")

        # Standard A31 improvement areas
        error_issues = [issue for issue in issues if issue.severity == QualitySeverity.ERROR]
        if error_issues:
            suggestions.append("Important A31 improvements needed:")
            for issue in error_issues[:3]:
                if issue.suggestion:
                    suggestions.append(f"Important: {issue.suggestion}")

        return suggestions

    def calculate_quality_trends(
        self, current_result: QualityCheckResult, historical_results: list[QualityCheckResult]
    ) -> dict[str, Any]:
        """Calculate quality trends based on historical data

        Args:
            current_result: Current quality result
            historical_results: Previous quality results

        Returns:
            dict[str, Any]: Quality trend analysis
        """
        if not historical_results:
            return {"trend": "no_data", "message": "No historical data available"}

        # Calculate trend
        recent_scores = [result.overall_percentage for result in historical_results[-5:]]
        current_score = current_result.overall_percentage

        if len(recent_scores) >= 2:
            avg_recent = sum(recent_scores) / len(recent_scores)
            if current_score > avg_recent + 5:
                trend = "improving"
            elif current_score < avg_recent - 5:
                trend = "declining"
            else:
                trend = "stable"
        else:
            trend = "insufficient_data"

        return {
            "trend": trend,
            "current_score": current_score,
            "average_recent": sum(recent_scores) / len(recent_scores) if recent_scores else 0,
            "historical_count": len(historical_results),
            "message": self._get_trend_message(trend),
        }

    def _generate_type_specific_suggestions(self, check_type: QualityCheckType, overall_percentage: float) -> list[str]:
        """Generate suggestions specific to check type

        Args:
            check_type: Type of quality check
            overall_percentage: Overall quality percentage

        Returns:
            list[str]: Type-specific suggestions
        """
        suggestions = []

        if check_type == QualityCheckType.COMPREHENSIVE:
            if overall_percentage < 75:
                suggestions.append("Focus on improving character development and narrative depth")
        elif check_type == QualityCheckType.ADAPTIVE:
            suggestions.append("Adaptive quality patterns are being learned for future improvements")
        elif check_type == QualityCheckType.VIEWPOINT_AWARE:
            suggestions.append("Viewpoint consistency has been evaluated against established patterns")
        elif check_type == QualityCheckType.A31_CHECKLIST:
            suggestions.extend(self._suggestion_templates["a31_specific"])

        return suggestions

    def _count_issues_by_severity(self, issues: list[QualityIssue]) -> dict[QualitySeverity, int]:
        """Count issues by severity level

        Args:
            issues: List of quality issues

        Returns:
            dict[QualitySeverity, int]: Issue counts by severity
        """
        counts = dict.fromkeys(QualitySeverity, 0)
        for issue in issues:
            counts[issue.severity] += 1
        return counts

    def _count_issues_by_type(self, issues: list[QualityIssue]) -> dict[str, int]:
        """Count issues by type

        Args:
            issues: List of quality issues

        Returns:
            dict[str, int]: Issue counts by type
        """
        counts = {}
        for issue in issues:
            counts[issue.type] = counts.get(issue.type, 0) + 1
        return counts

    def _get_trend_message(self, trend: str) -> str:
        """Get descriptive message for trend

        Args:
            trend: Trend type

        Returns:
            str: Trend description
        """
        messages: dict[str, Any] = {
            "improving": "Quality is improving compared to recent episodes",
            "declining": "Quality has declined compared to recent episodes",
            "stable": "Quality is consistent with recent episodes",
            "insufficient_data": "Not enough data to determine trend",
            "no_data": "No historical data available for comparison",
        }
        return messages.get(trend, "Unknown trend")

    def _load_grade_thresholds(self) -> dict[str, float]:
        """Load quality grade thresholds

        Returns:
            dict[str, float]: Grade thresholds
        """
        return {"A+": 90.0, "A": 85.0, "B+": 80.0, "B": 75.0, "C+": 70.0, "C": 65.0, "D+": 60.0, "D": 55.0}

    def _load_suggestion_templates(self) -> dict[str, list[str]]:
        """Load suggestion templates

        Returns:
            dict[str, list[str]]: Suggestion templates
        """
        return {
            "low_quality": [
                "Consider substantial revision focusing on core narrative elements",
                "Review basic writing fundamentals and story structure",
            ],
            "medium_quality": [
                "Focus on enhancing character development and emotional depth",
                "Consider improving dialogue balance and pacing",
            ],
            "high_quality": [
                "Fine-tune narrative elements for optimal impact",
                "Consider advanced techniques for enhanced reader engagement",
            ],
            "a31_specific": [
                "Refer to A31 manuscript execution checklist for detailed criteria",
                "Focus on opening appeal and character growth elements",
            ],
        }



#!/usr/bin/env python3
"""学習機能付き品質チェックドメイン - 値オブジェクトのユニットテスト

TDD原則に従い、値オブジェクトの不変性とビジネスルールをテスト


仕様書: SPEC-UNIT-TEST
"""

import pytest

from noveler.domain.learning.value_objects import (
    CorrelationInsight,
    LearningDataQuality,
    QualityEvaluationResult,
    QualityMetric,
    WritingStyleProfile,
)
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


class TestQualityMetric:
    """QualityMetric列挙型のテスト"""

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-ALL_METRICS_DEFINED")
    def test_all_metrics_defined(self) -> None:
        """全てのメトリックが定義されていることを確認"""
        expected_metrics = [
            "readability",
            "dialogue_ratio",
            "sentence_variety",
            "narrative_depth",
            "emotional_intensity",
        ]

        actual_metrics = [metric.value for metric in QualityMetric]

        assert set(expected_metrics) == set(actual_metrics)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-METRIC_VALUES")
    def test_metric_values(self) -> None:
        """メトリック値の確認"""
        assert QualityMetric.READABILITY.value == "readability"
        assert QualityMetric.DIALOGUE_RATIO.value == "dialogue_ratio"
        assert QualityMetric.SENTENCE_VARIETY.value == "sentence_variety"
        assert QualityMetric.NARRATIVE_DEPTH.value == "narrative_depth"
        assert QualityMetric.EMOTIONAL_INTENSITY.value == "emotional_intensity"


class TestLearningDataQuality:
    """LearningDataQuality列挙型のテスト"""

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-ALL_QUALITY_LEVELS_D")
    def test_all_quality_levels_defined(self) -> None:
        """全ての品質レベルが定義されていることを確認"""
        expected_levels = ["insufficient", "low", "medium", "high", "excellent"]

        actual_levels = [level.value for level in LearningDataQuality]

        assert set(expected_levels) == set(actual_levels)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-QUALITY_LEVEL_VALUES")
    def test_quality_level_values(self) -> None:
        """品質レベル値の確認"""
        assert LearningDataQuality.INSUFFICIENT.value == "insufficient"
        assert LearningDataQuality.LOW.value == "low"
        assert LearningDataQuality.MEDIUM.value == "medium"
        assert LearningDataQuality.HIGH.value == "high"
        assert LearningDataQuality.EXCELLENT.value == "excellent"


class TestWritingStyleProfile:
    """WritingStyleProfile値オブジェクトのテスト"""

    def setup_method(self) -> None:
        """各テストメソッドの前に実行"""
        self.valid_features = {
            "avg_sentence_length": 35.0,
            "dialogue_ratio": 0.4,
            "emotional_words_ratio": 0.15,
            "comma_frequency": 0.3,
        }
        self.timestamp = project_now().datetime

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CREATE_VALID_PROFILE")
    def test_create_valid_profile(self) -> None:
        """有効なプロファイルの作成"""
        # When
        profile = WritingStyleProfile(
            profile_id="test_profile",
            features=self.valid_features,
            confidence_score=0.85,
            sample_count=20,
            last_updated=self.timestamp,
        )

        # Then
        assert profile.profile_id == "test_profile"
        assert profile.features == self.valid_features
        assert profile.confidence_score == 0.85
        assert profile.sample_count == 20
        assert profile.last_updated == self.timestamp

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IMMUTABILITY")
    def test_immutability(self) -> None:
        """不変性の確認"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test_profile",
            features=self.valid_features,
            confidence_score=0.85,
            sample_count=20,
            last_updated=self.timestamp,
        )

        # When & Then
        with pytest.raises(AttributeError, match=".*"):
            profile.profile_id = "new_id"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_RATIO_FEATU")
    def test_validate_ratio_feature_valid(self) -> None:
        """比率特徴量の有効範囲検証(正常)"""
        # Given
        features = {
            "dialogue_ratio": 0.5,
            "emotional_words_ratio": 0.2,
        }

        # When & Then (例外が発生しない)
        WritingStyleProfile(
            profile_id="test",
            features=features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_RATIO_FEATU")
    def test_validate_ratio_feature_invalid(self) -> None:
        """比率特徴量の有効範囲検証(異常)"""
        # Given
        features = {"dialogue_ratio": 1.5}  # 範囲外

        # When & Then
        with pytest.raises(ValueError) as exc_info:
            WritingStyleProfile(
                profile_id="test",
                features=features,
                confidence_score=0.8,
                sample_count=10,
                last_updated=self.timestamp,
            )

        assert "特徴量が有効範囲外: dialogue_ratio=1.5" in str(exc_info.value)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_SENTENCE_LE")
    def test_validate_sentence_length_valid(self) -> None:
        """文長特徴量の有効範囲検証(正常)"""
        # Given
        features = {"avg_sentence_length": 50.0}

        # When & Then (例外が発生しない)
        WritingStyleProfile(
            profile_id="test",
            features=features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_SENTENCE_LE")
    def test_validate_sentence_length_invalid(self) -> None:
        """文長特徴量の有効範囲検証(異常)"""
        # Given
        features = {"avg_sentence_length": 250.0}  # 範囲外

        # When & Then
        with pytest.raises(ValueError) as exc_info:
            WritingStyleProfile(
                profile_id="test",
                features=features,
                confidence_score=0.8,
                sample_count=10,
                last_updated=self.timestamp,
            )

        assert "特徴量が有効範囲外: avg_sentence_length=250.0" in str(exc_info.value)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_CONFIDENCE_")
    def test_validate_confidence_valid(self) -> None:
        """信頼度の有効範囲検証(正常)"""
        # When & Then (例外が発生しない)
        WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=1.0,
            sample_count=10,
            last_updated=self.timestamp,
        )

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-VALIDATE_CONFIDENCE_")
    def test_validate_confidence_invalid(self) -> None:
        """信頼度の有効範囲検証(異常)"""
        # When & Then
        with pytest.raises(ValueError) as exc_info:
            WritingStyleProfile(
                profile_id="test",
                features=self.valid_features,
                confidence_score=1.5,  # 範囲外
                sample_count=10,
                last_updated=self.timestamp,
            )

        assert "信頼度が有効範囲外: 1.5" in str(exc_info.value)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_FEATURE_EXISTING")
    def test_get_feature_existing(self) -> None:
        """存在する特徴量の取得"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        value = profile.get_feature("dialogue_ratio")

        # Then
        assert value == 0.4

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_FEATURE_NON_EXIS")
    def test_get_feature_non_existing(self) -> None:
        """存在しない特徴量の取得"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        value = profile.get_feature("unknown_feature")

        # Then
        assert value == 0.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_RELIABLE_TRUE")
    def test_is_reliable_true(self) -> None:
        """信頼度十分チェック(True)"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.85,
            sample_count=15,
            last_updated=self.timestamp,
        )

        # When & Then
        assert profile.is_reliable() is True

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_RELIABLE_LOW_CONF")
    def test_is_reliable_low_confidence(self) -> None:
        """信頼度十分チェック(信頼度不足)"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.7,  # 閾値未満
            sample_count=15,
            last_updated=self.timestamp,
        )

        # When & Then
        assert profile.is_reliable() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_RELIABLE_LOW_SAMP")
    def test_is_reliable_low_sample_count(self) -> None:
        """信頼度十分チェック(サンプル数不足)"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.85,
            sample_count=5,  # 閾値未満
            last_updated=self.timestamp,
        )

        # When & Then
        assert profile.is_reliable() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CALCULATE_SIMILARITY")
    def test_calculate_similarity_identical(self) -> None:
        """類似度計算(同一プロファイル)"""
        # Given
        profile1 = WritingStyleProfile(
            profile_id="test1",
            features=self.valid_features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        profile2 = WritingStyleProfile(
            profile_id="test2",
            features=self.valid_features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        similarity = profile1.calculate_similarity(profile2)

        # Then
        assert similarity == pytest.approx(1.0, abs=0.001)

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CALCULATE_SIMILARITY")
    def test_calculate_similarity_different(self) -> None:
        """類似度計算(異なるプロファイル)"""
        # Given
        profile1 = WritingStyleProfile(
            profile_id="test1",
            features={"dialogue_ratio": 0.2, "emotional_words_ratio": 0.1},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        profile2 = WritingStyleProfile(
            profile_id="test2",
            features={"dialogue_ratio": 0.8, "emotional_words_ratio": 0.9},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        similarity = profile1.calculate_similarity(profile2)

        # Then
        assert 0 <= similarity <= 1

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CALCULATE_SIMILARITY")
    def test_calculate_similarity_no_common_features(self) -> None:
        """類似度計算(共通特徴量なし)"""
        # Given
        profile1 = WritingStyleProfile(
            profile_id="test1",
            features={"dialogue_ratio": 0.4},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        profile2 = WritingStyleProfile(
            profile_id="test2",
            features={"emotional_words_ratio": 0.2},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        similarity = profile1.calculate_similarity(profile2)

        # Then
        assert similarity == 0.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CALCULATE_SIMILARITY")
    def test_calculate_similarity_zero_magnitude(self) -> None:
        """類似度計算(ゼロベクトル)"""
        # Given
        profile1 = WritingStyleProfile(
            profile_id="test1",
            features={"dialogue_ratio": 0.0},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        profile2 = WritingStyleProfile(
            profile_id="test2",
            features={"dialogue_ratio": 0.5},
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        similarity = profile1.calculate_similarity(profile2)

        # Then
        assert similarity == 0.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_FEATURE_IMPORTAN")
    def test_get_feature_importance_weights(self) -> None:
        """特徴量重要度ウェイト取得"""
        # Given
        profile = WritingStyleProfile(
            profile_id="test",
            features=self.valid_features,
            confidence_score=0.8,
            sample_count=10,
            last_updated=self.timestamp,
        )

        # When
        weights = profile.get_feature_importance_weights()

        # Then
        assert "avg_sentence_length" in weights
        assert "dialogue_ratio" in weights
        # 信頼度による調整確認
        assert weights["avg_sentence_length"] == pytest.approx(0.2 * 0.8)


class TestQualityEvaluationResult:
    """QualityEvaluationResult値オブジェクトのテスト"""

    def setup_method(self) -> None:
        """各テストメソッドの前に実行"""
        self.metric_scores = {
            QualityMetric.READABILITY: 85.0,
            QualityMetric.DIALOGUE_RATIO: 80.0,
            QualityMetric.NARRATIVE_DEPTH: 82.0,
        }
        self.timestamp = project_now().datetime

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CREATE_EVALUATION_RE")
    def test_create_evaluation_result(self) -> None:
        """評価結果の作成"""
        # When
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={"readability_weight": 1.1},
            evaluation_timestamp=self.timestamp,
        )

        # Then
        assert result.total_score == 82.33
        assert result.metric_scores == self.metric_scores
        assert result.confidence_level == 0.85
        assert result.personalized_adjustments == {"readability_weight": 1.1}
        assert result.evaluation_timestamp == self.timestamp

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IMMUTABILITY")
    def test_immutability(self) -> None:
        """不変性の確認"""
        # Given
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        with pytest.raises(AttributeError, match=".*"):
            result.total_score = 90.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-HAS_PERSONALIZED_ADJ")
    def test_has_personalized_adjustments_true(self) -> None:
        """個人化調整有無チェック(あり)"""
        # Given
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={"readability_weight": 1.1},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        assert result.has_personalized_adjustments() is True

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-HAS_PERSONALIZED_ADJ")
    def test_has_personalized_adjustments_false(self) -> None:
        """個人化調整有無チェック(なし)"""
        # Given
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        assert result.has_personalized_adjustments() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_METRIC_SCORE_EXI")
    def test_get_metric_score_existing(self) -> None:
        """メトリック別スコア取得(存在)"""
        # Given
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When
        score = result.get_metric_score(QualityMetric.READABILITY)

        # Then
        assert score == 85.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_METRIC_SCORE_NON")
    def test_get_metric_score_non_existing(self) -> None:
        """メトリック別スコア取得(非存在)"""
        # Given
        result = QualityEvaluationResult(
            total_score=82.33,
            metric_scores=self.metric_scores,
            confidence_level=0.85,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When
        score = result.get_metric_score(QualityMetric.SENTENCE_VARIETY)

        # Then
        assert score == 0.0

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_HIGH_QUALITY_TRUE")
    def test_is_high_quality_true(self) -> None:
        """高品質判定(True)"""
        # Given
        result = QualityEvaluationResult(
            total_score=85.0,
            metric_scores=self.metric_scores,
            confidence_level=0.8,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        assert result.is_high_quality() is True

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_HIGH_QUALITY_LOW_")
    def test_is_high_quality_low_score(self) -> None:
        """高品質判定(スコア不足)"""
        # Given
        result = QualityEvaluationResult(
            total_score=75.0,  # 閾値未満
            metric_scores=self.metric_scores,
            confidence_level=0.8,
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        assert result.is_high_quality() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_HIGH_QUALITY_LOW_")
    def test_is_high_quality_low_confidence(self) -> None:
        """高品質判定(信頼度不足)"""
        # Given
        result = QualityEvaluationResult(
            total_score=85.0,
            metric_scores=self.metric_scores,
            confidence_level=0.6,  # 閾値未満
            personalized_adjustments={},
            evaluation_timestamp=self.timestamp,
        )

        # When & Then
        assert result.is_high_quality() is False


class TestCorrelationInsight:
    """CorrelationInsight値オブジェクトのテスト"""

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-CREATE_CORRELATION_I")
    def test_create_correlation_insight(self) -> None:
        """相関分析洞察の作成"""
        # When
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.75,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="可読性の向上が読者評価向上につながる",
        )

        # Then
        assert insight.metric_pair == ("readability", "reader_rating")
        assert insight.correlation_coefficient == 0.75
        assert insight.significance_level == 0.01
        assert insight.sample_size == 50
        assert insight.actionable_insight == "可読性の向上が読者評価向上につながる"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IMMUTABILITY")
    def test_immutability(self) -> None:
        """不変性の確認"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.75,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="可読性の向上が読者評価向上につながる",
        )

        # When & Then
        with pytest.raises(AttributeError, match=".*"):
            insight.correlation_coefficient = 0.8

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_SIGNIFICANT_TRUE")
    def test_is_significant_true(self) -> None:
        """統計的有意性チェック(有意)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.5,
            significance_level=0.01,  # < 0.05
            sample_size=50,
            actionable_insight="相関あり",
        )

        # When & Then
        assert insight.is_significant() is True

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_SIGNIFICANT_HIGH_")
    def test_is_significant_high_p_value(self) -> None:
        """統計的有意性チェック(p値が高い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.5,
            significance_level=0.1,  # > 0.05
            sample_size=50,
            actionable_insight="相関あり",
        )

        # When & Then
        assert insight.is_significant() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-IS_SIGNIFICANT_LOW_C")
    def test_is_significant_low_correlation(self) -> None:
        """統計的有意性チェック(相関係数が低い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.2,  # < 0.3
            significance_level=0.01,
            sample_size=50,
            actionable_insight="弱い相関",
        )

        # When & Then
        assert insight.is_significant() is False

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_very_strong(self) -> None:
        """相関の強さ説明(非常に強い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.85,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="強い相関",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "非常に強い"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_strong(self) -> None:
        """相関の強さ説明(強い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.65,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="相関あり",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "強い"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_moderate(self) -> None:
        """相関の強さ説明(中程度)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.45,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="相関あり",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "中程度"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_weak(self) -> None:
        """相関の強さ説明(弱い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.25,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="弱い相関",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "弱い"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_very_weak(self) -> None:
        """相関の強さ説明(非常に弱い)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=0.1,
            significance_level=0.01,
            sample_size=50,
            actionable_insight="ほぼ相関なし",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "非常に弱い"

    @pytest.mark.spec("SPEC-LEARNING_VALUE_OBJECTS-GET_STRENGTH_DESCRIP")
    def test_get_strength_description_negative_correlation(self) -> None:
        """相関の強さ説明(負の相関)"""
        # Given
        insight = CorrelationInsight(
            metric_pair=("readability", "reader_rating"),
            correlation_coefficient=-0.75,  # 負の値
            significance_level=0.01,
            sample_size=50,
            actionable_insight="負の相関",
        )

        # When
        description = insight.get_strength_description()

        # Then
        assert description == "強い"  # 絶対値で判定される
